@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2017-2020 darktable developers.
+    Copyright (C) 2017-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,9 +21,6 @@
 #include "control/control.h"
 #include "develop/imageop.h"
 #include "dwt.h"
-#if defined(__SSE__)
-#include <xmmintrin.h>
-#endif
 
 /* Based on the original source code of GIMP's Wavelet Decompose plugin, by Marco Rossini
  *
@@ -33,7 +30,7 @@
 
 dwt_params_t *dt_dwt_init(float *image, const int width, const int height, const int ch, const int scales,
                           const int return_layer, const int merge_from_scale, void *user_data,
-                          const float preview_scale, const int use_sse)
+                          const float preview_scale)
 {
   dwt_params_t *p = (dwt_params_t *)malloc(sizeof(dwt_params_t));
   if(!p) return NULL;
@@ -47,7 +44,6 @@ dwt_params_t *dt_dwt_init(float *image, const int width, const int height, const
   p->merge_from_scale = merge_from_scale;
   p->user_data = user_data;
   p->preview_scale = preview_scale;
-  p->use_sse = use_sse;
 
   return p;
 }
@@ -109,7 +105,8 @@ int dt_dwt_first_scale_visible(dwt_params_t *p)
 
 static void dwt_get_image_layer(float *const layer, dwt_params_t *const p)
 {
-  if(p->image != layer) memcpy(p->image, layer, sizeof(float) * p->width * p->height * p->ch);
+  if(p->image != layer)
+    dt_iop_image_copy_by_size(p->image, layer, p->width, p->height, p->ch);
 }
 
 // first, "vertical" pass of wavelet decomposition
@@ -150,14 +147,19 @@ static void dwt_decompose_vert(float *const restrict out, const float *const res
 
 // second, horizontal pass of wavelet decomposition; generates 'coarse' into the output buffer and overwrites
 //   the input buffer with 'details'
-static void dwt_decompose_horiz(float *const restrict out, float *const restrict in, float *const temp,
-                                const size_t height, const size_t width, const size_t lev)
+static void dwt_decompose_horiz(
+    float *const restrict out,
+    float *const restrict in,
+    float *const temp,
+    size_t padded_size,
+    const size_t height,
+    const size_t width,
+    const size_t lev)
 {
   const int hscale = MIN(1 << lev, width);  //(int because we need a signed difference below)
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-  dt_omp_firstprivate(height, width, hscale) \
-  dt_omp_sharedconst(in, out, temp) \
+  dt_omp_firstprivate(height, width, hscale, in, out, temp, padded_size)    \
   schedule(static)
 #endif
   for(int row = 0; row < height ; row++)
@@ -168,7 +170,7 @@ static void dwt_decompose_horiz(float *const restrict out, float *const restrict
     // final sum and split the original input into 'coarse' and 'details' by subtracting the scaled sum from
     // the original input.
     const size_t rowindex = (size_t)4 * (row * width);
-    float* const restrict temprow = temp + width * dt_get_thread_num() * 4;
+    float* const restrict temprow = dt_get_perthread(temp,padded_size);
     float* const restrict details = in + rowindex;
     float* const restrict coarse = out + rowindex;
 
@@ -210,64 +212,63 @@ static void dwt_decompose_horiz(float *const restrict out, float *const restrict
 }
 
 // split input into 'coarse' and 'details'; put 'details' back into the input buffer
-static void dwt_decompose_layer(float *const restrict out, float *const restrict in, float *const temp, const int lev,
-                                const dwt_params_t *const p)
+static void dwt_decompose_layer(
+    float *const restrict out,
+    float *const restrict in,
+    float *const temp,
+    size_t padded_size,
+    const int lev,
+    const dwt_params_t *const p)
 {
   dwt_decompose_vert(out, in, p->height, p->width, lev);
-  dwt_decompose_horiz(out, in, temp, p->height, p->width, lev);
+  dwt_decompose_horiz(out, in, temp, padded_size, p->height, p->width, lev);
   return;
 }
 
 /* actual decomposing algorithm */
-static void dwt_wavelet_decompose(float *img, dwt_params_t *const p, _dwt_layer_func layer_func)
+static void dwt_wavelet_decompose(float *img,
+                                  dwt_params_t *const p,
+                                  _dwt_layer_func layer_func)
 {
-  float *temp = NULL;
-  float *layers = NULL;
+  assert(p->ch == 4);
+
+  float *temp = NULL;		// scratch buffer for decomposition
+  float *layers = NULL;		// buffer to reconstruct the image
   float *merged_layers = NULL;
   float *buffer[2] = { 0, 0 };
-  int bcontinue = 1;
-  const size_t size = (size_t)p->width * p->height * p->ch;
-
-  assert(p->ch == 4);
 
   if(layer_func) layer_func(img, p, 0);
 
-  if(p->scales <= 0) goto cleanup;
+  if(p->scales <= 0)
+    return;
 
   /* image buffers */
   buffer[0] = img;
-  /* temporary storage */
-  buffer[1] = dt_alloc_align_float(size);
-  // buffer to reconstruct the image
-  layers = dt_alloc_align_float((size_t)4 * p->width * p->height);
-  // scratch buffer for decomposition
-  temp = dt_alloc_align_float(dt_get_num_threads() * 4 * p->width);
 
-  if(buffer[1] == NULL || layers == NULL || temp == NULL)
+  /* allocate temporary storage */
+  dt_iop_roi_t roi = { .x = 0, .y = 0, .height = p->height, .width = p->width };
+  size_t padded_size;
+  const int do_merge = p->merge_from_scale > 0;
+  if (!dt_iop_alloc_image_buffers(NULL, &roi, &roi,
+                                  4 | DT_IMGSZ_INPUT, &buffer[1],
+                                  4 | DT_IMGSZ_INPUT | DT_IMGSZ_CLEARBUF, &layers,
+                                  4 | DT_IMGSZ_WIDTH | DT_IMGSZ_PERTHREAD, &temp, &padded_size,
+                                  (do_merge ? 4 | DT_IMGSZ_INPUT | DT_IMGSZ_CLEARBUF : 0), &merged_layers,
+                                  0, NULL))
   {
-    printf("not enough memory for wavelet decomposition");
-    goto cleanup;
-  }
-  dt_iop_image_fill(layers,0.0f,p->width,p->height,p->ch);
-
-  if(p->merge_from_scale > 0)
-  {
-    merged_layers = dt_alloc_align_float((size_t)p->width * p->height * p->ch);
-    if(merged_layers == NULL)
-    {
-      printf("not enough memory for wavelet decomposition");
-      goto cleanup;
-    }
-    dt_iop_image_fill(merged_layers,0.0f,p->width,p->height,p->ch);
+    dt_print(DT_DEBUG_ALWAYS,
+             "[dwt] unable to alloc working memory, skipping wavelet decomposition\n");
+    return;
   }
 
   // iterate over wavelet scales
   unsigned int hpass = 0;
+  int bcontinue = 1;
   for(unsigned int lev = 0; lev < p->scales && bcontinue; lev++)
   {
     unsigned int lpass = (1 - (lev & 1));
 
-    dwt_decompose_layer(buffer[lpass], buffer[hpass], temp, lev, p);
+    dwt_decompose_layer(buffer[lpass], buffer[hpass], temp, padded_size, lev, p);
 
     // no merge scales or we didn't reach the merge scale from yet
     if(p->merge_from_scale == 0 || p->merge_from_scale > lev + 1)
@@ -345,11 +346,11 @@ static void dwt_wavelet_decompose(float *img, dwt_params_t *const p, _dwt_layer_
     }
   }
 
-cleanup:
-  if(temp) dt_free_align(temp);
-  if(layers) dt_free_align(layers);
-  if(merged_layers) dt_free_align(merged_layers);
-  if(buffer[1]) dt_free_align(buffer[1]);
+  dt_free_align(temp);
+  dt_free_align(layers);
+  dt_free_align(buffer[1]);
+  if(merged_layers)
+    dt_free_align(merged_layers);
 }
 
 /* this function prepares for decomposing, which is done in the function dwt_wavelet_decompose() */
@@ -383,8 +384,12 @@ void dwt_decompose(dwt_params_t *p, _dwt_layer_func layer_func)
 }
 
 // first, "vertical" pass of wavelet decomposition
-static void dwt_denoise_vert_1ch(float *const restrict out, const float *const restrict in,
-                                 const size_t height, const size_t width, const size_t lev)
+static void dwt_denoise_vert_1ch(
+    float *const restrict out,
+    const float *const restrict in,
+    const size_t height,
+    const size_t width,
+    const size_t lev)
 {
   const int vscale = MIN(1 << lev, height);
 #ifdef _OPENMP
@@ -419,9 +424,15 @@ static void dwt_denoise_vert_1ch(float *const restrict out, const float *const r
 
 // second, horizontal pass of wavelet decomposition; generates 'coarse' into the output buffer and overwrites
 //   the input buffer with 'details'
-static void dwt_denoise_horiz_1ch(float *const restrict out, float *const restrict in,
-                                  float *const restrict accum, const size_t height, const size_t width,
-                                  const size_t lev, const float thold, const int last)
+static void dwt_denoise_horiz_1ch(
+    float *const restrict out,
+    float *const restrict in,
+    float *const restrict accum,
+    const size_t height,
+    const size_t width,
+    const size_t lev,
+    const float thold,
+    const int last)
 {
   const int hscale = MIN(1 << lev, width);
 #ifdef _OPENMP
@@ -504,9 +515,18 @@ static void dwt_denoise_horiz_1ch(float *const restrict out, float *const restri
  * recomposing the result from just the portion of each scale which exceeds the magnitude of the given
  * threshold for that scale.
  */
-void dwt_denoise(float *const img, const int width, const int height, const int bands, const float *const noise)
+void dwt_denoise(float *const img,
+                 const int width,
+                 const int height,
+                 const int bands,
+                 const float *const noise)
 {
   float *const details = dt_alloc_align_float((size_t)2 * width * height);
+  if(!details)
+  {
+    dt_print(DT_DEBUG_ALWAYS,"[dwt_denoise] unable to alloc working memory, skipping denoise\n");
+    return;
+  }
   float *const interm = details + width * height;	// temporary storage for use during each pass
 
   // zero the accumulator
@@ -516,11 +536,14 @@ void dwt_denoise(float *const img, const int width, const int height, const int
   {
     const int last = (lev+1) == bands;
 
-    // "vertical" pass, averages pixels with those 'scale' rows above and below and puts result in 'interm'
+    // "vertical" pass, averages pixels with those 'scale' rows above
+    // and below and puts result in 'interm'
     dwt_denoise_vert_1ch(interm, img, height, width, lev);
-    // horizontal filtering pass, averages pixels in 'interm' with those 'scale' rows to the left and right
-    // accumulates the portion of the detail scale that is above the noise threshold into 'details'; this
-    // will be added to the residue left in 'img' on the last iteration
+    // horizontal filtering pass, averages pixels in 'interm' with
+    // those 'scale' rows to the left and right
+    // accumulates the portion of the detail scale that is above the
+    // noise threshold into 'details'; this will be added to the
+    // residue left in 'img' on the last iteration
     dwt_denoise_horiz_1ch(interm, img, details, height, width, lev, noise[lev], last);
   }
   dt_free_align(details);
@@ -669,7 +692,7 @@ static cl_int dwt_wavelet_decompose_cl(cl_mem img, dwt_params_cl_t *const p, _dw
   buffer[1] = dt_opencl_alloc_device_buffer(devid, sizeof(float) * p->ch * p->width * p->height);
   if(buffer[1] == NULL)
   {
-    printf("not enough memory for wavelet decomposition");
+    dt_print(DT_DEBUG_ALWAYS, "[dwt] not enough memory for wavelet decomposition\n");
     err = CL_MEM_OBJECT_ALLOCATION_FAILURE;
     goto cleanup;
   }
@@ -678,7 +701,7 @@ static cl_int dwt_wavelet_decompose_cl(cl_mem img, dwt_params_cl_t *const p, _dw
   layers = dt_opencl_alloc_device_buffer(devid, sizeof(float) * p->ch * p->width * p->height);
   if(layers == NULL)
   {
-    printf("not enough memory for wavelet decomposition");
+    dt_print(DT_DEBUG_ALWAYS, "[dwt] not enough memory for wavelet decomposition\n");
     err = CL_MEM_OBJECT_ALLOCATION_FAILURE;
     goto cleanup;
   }
@@ -699,7 +722,7 @@ static cl_int dwt_wavelet_decompose_cl(cl_mem img, dwt_params_cl_t *const p, _dw
     merged_layers = dt_opencl_alloc_device_buffer(devid, sizeof(float) * p->ch * p->width * p->height);
     if(merged_layers == NULL)
     {
-      printf("not enough memory for wavelet decomposition");
+      dt_print(DT_DEBUG_ALWAYS, "[dwt] not enough memory for wavelet decomposition\n");
       err = CL_MEM_OBJECT_ALLOCATION_FAILURE;
       goto cleanup;
     }
@@ -728,7 +751,7 @@ static cl_int dwt_wavelet_decompose_cl(cl_mem img, dwt_params_cl_t *const p, _dw
     temp = dt_opencl_alloc_device_buffer(devid, sizeof(float) * p->ch * p->width * p->height);
     if(temp == NULL)
     {
-      printf("not enough memory for wavelet decomposition");
+      dt_print(DT_DEBUG_ALWAYS, "[dwt] not enough memory for wavelet decomposition\n");
       err = CL_MEM_OBJECT_ALLOCATION_FAILURE;
       goto cleanup;
     }
@@ -917,4 +940,3 @@ cl_int dwt_decompose_cl(dwt_params_cl_t *p, _dwt_layer_func_cl layer_func)
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
 // clang-format on
-
