@@ -22,6 +22,12 @@
 #include <sqlite3.h>
 #include <stdint.h>
 
+#include "common/darktable.h"
+
+#ifdef __cplusplus
+extern "C" {
+#endif /* __cplusplus */
+
 typedef struct dt_tag_t
 {
   guint id;
@@ -91,10 +97,10 @@ gboolean dt_tag_exists(const char *name, guint *tagid);
 gboolean dt_tag_attach_images(const guint tagid, const GList *img, const gboolean undo_on);
 /** attach a tag on images. tagid id of tag to attach. imgid the image
  * id to attach tag to, if < 0 images to act on are used. */
-gboolean dt_tag_attach(const guint tagid, const gint imgid, const gboolean undo_on, const gboolean group_on);
+gboolean dt_tag_attach(const guint tagid, const dt_imgid_t imgid, const gboolean undo_on, const gboolean group_on);
 
 /** check if a tag is attached to the given image */
-gboolean dt_is_tag_attached(const guint tagid, const gint imgid);
+gboolean dt_is_tag_attached(const guint tagid, const dt_imgid_t imgid);
 
 /** attach a list of tags on selected images. \param[in] tags a list of ids of tags. \param[in] imgid the
  * image id to attach tag to, if < 0 selected images are used. \note If tag not exists it's created
@@ -111,14 +117,14 @@ gboolean dt_tag_attach_string_list(const gchar *tags, const GList *img, const gb
 gboolean dt_tag_detach_images(const guint tagid, const GList *img, const gboolean undo_on);
 /** detach tag from images. \param[in] tagid of tag to detach. \param[in] imgid the image id to detach
  * tag from, if < 0 images to act on are used. */
-gboolean dt_tag_detach(const guint tagid, const gint imgid, const gboolean undo_on, const gboolean group_on);
+gboolean dt_tag_detach(const guint tagid, const dt_imgid_t imgid, const gboolean undo_on, const gboolean group_on);
 
 /** detach tags from images that matches name, it is valid to use % to match tag */
-gboolean dt_tag_detach_by_string(const char *name, const gint imgid, const gboolean undo_on,
+gboolean dt_tag_detach_by_string(const char *name, const dt_imgid_t imgid, const gboolean undo_on,
                                  const gboolean group_on);
 
 /** retrieves a list of tags of specified imgid \param[out] result a list of dt_tag_t, sorted by tag. */
-uint32_t dt_tag_get_attached(const gint imgid, GList **result, const gboolean ignore_dt_tags);
+uint32_t dt_tag_get_attached(const dt_imgid_t imgid, GList **result, const gboolean ignore_dt_tags);
 
 /** sort tags per name (including '|') or per count (desc) */
 GList *dt_sort_tag(GList *tags, gboolean byname);
@@ -126,22 +132,22 @@ GList *dt_sort_tag(GList *tags, gboolean byname);
 /** get a list of tags,
  *  the difference to dt_tag_get_attached() is that this one splits at '|' and filters out the "darktable|"
  * tags. */
-GList *dt_tag_get_list(gint imgid);
+GList *dt_tag_get_list(dt_imgid_t imgid);
 
 /** get a list of tags,
  *  the difference to dt_tag_get_list() is that this one checks option for exportation */
-GList *dt_tag_get_list_export(gint imgid, int32_t flags);
+GList *dt_tag_get_list_export(dt_imgid_t imgid, int32_t flags);
 
 /** get a flat list of only hierarchical tags,
  *  the difference to dt_tag_get_attached() is that this one filters out the "darktable|" tags. */
-GList *dt_tag_get_hierarchical(gint imgid);
+GList *dt_tag_get_hierarchical(dt_imgid_t imgid);
 
 /** get a flat list of only hierarchical tags,
  *  the difference to dt_tag_get_hierarchical() is that this one checks option for exportation */
-GList *dt_tag_get_hierarchical_export(gint imgid, int32_t flags);
+GList *dt_tag_get_hierarchical_export(dt_imgid_t imgid, int32_t flags);
 
 /** get a flat list of tags id attached to image id*/
-GList *dt_tag_get_tags(const gint imgid, const gboolean ignore_dt_tags);
+GList *dt_tag_get_tags(const dt_imgid_t imgid, const gboolean ignore_dt_tags);
 
 /** get the subset of images that have a given tag attached */
 GList *dt_tag_get_images(const gint tagid);
@@ -193,15 +199,7 @@ uint32_t dt_selected_images_count();
 uint32_t dt_tag_images_count(gint tagid);
 
 /** retrieves the subtags of requested level for the requested category */
-char *dt_tag_get_subtags(const gint imgid, const char *category, const int level);
-
-/** return the images order associated to that tag */
-gboolean dt_tag_get_tag_order_by_id(const uint32_t tagid, uint32_t *sort,
-                                          gboolean *descending);
-
-/** save the images order on the tag */
-void dt_tag_set_tag_order_by_id(const uint32_t tagid, const uint32_t sort,
-                                const gboolean descending);
+char *dt_tag_get_subtags(const dt_imgid_t imgid, const char *category, const int level);
 
 /** return the tagid of that tag - follow tag sensitivity - return 0 if not found*/
 uint32_t dt_tag_get_tag_id_by_name(const char * const name);
@@ -209,9 +207,12 @@ uint32_t dt_tag_get_tag_id_by_name(const char * const name);
 /** init the darktable tags table */
 void dt_set_darktable_tags();
 
+#ifdef __cplusplus
+} // extern "C"
+#endif /* __cplusplus */
+
 // clang-format off
 // modelines: These editor modelines have been set for all relevant files by tools/update_modelines.py
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
 // clang-format on
-
