@@ -19,6 +19,7 @@
 */
 
 #include "decompressors/PhaseOneDecompressor.h" // for PhaseOneDecompressor
+#include "MemorySanitizer.h"                    // for MSan
 #include "common/RawImage.h"                    // for RawImage, RawImageData
 #include "common/RawspeedException.h"           // for RawspeedException
 #include "fuzz/Common.h"                        // for CreateRawImage
@@ -30,6 +31,7 @@
 #include <cstdint>         // for uint8_t
 #include <cstdio>          // for size_t
 #include <iterator>        // for back_insert_iterator, back_ins...
+#include <utility>         // for move
 
 extern "C" int LLVMFuzzerTestOneInput(const uint8_t* Data, size_t Size);
 
@@ -57,7 +59,8 @@ extern "C" int LLVMFuzzerTestOneInput(const uint8_t* Data, size_t Size) {
     mRaw->createData();
     f.decompress();
 
-    mRaw->checkMemIsInitialized();
+    rawspeed::MSan::CheckMemIsInitialized(
+        mRaw->getByteDataAsUncroppedArray2DRef());
   } catch (const rawspeed::RawspeedException&) {
     // Exceptions are good, crashes are bad.
   }
