@@ -21,28 +21,29 @@
 
 #pragma once
 
-#include "adt/Point.h"                       // for iPoint2D
-#include "adt/iterator_range.h"              // for iterator_range
-#include "common/RawImage.h"                 // for RawImage
-#include "common/RawspeedException.h"        // for ThrowException
-#include "decoders/RawDecoderException.h"    // for ThrowException, ThrowRDE
-#include "decompressors/DummyHuffmanTable.h" // for DummyHuffmanTable
-#include "decompressors/HuffmanTable.h" // for HuffmanTable, HuffmanTableLUT
-#include "io/ByteStream.h"              // for ByteStream
-#include <array>                        // for array
-#include <cassert>                      // for assert
-#include <cstddef>                      // for size_t
-#include <cstdint>                      // for uint16_t
-#include <functional>                   // for reference_wrapper
-#include <tuple>                        // for array, tuple
-#include <utility>                      // for index_sequence
-#include <vector>                       // for vector
+#include "adt/Invariant.h"                // for invariant
+#include "adt/Point.h"                    // for iPoint2D
+#include "adt/iterator_range.h"           // for iterator_range
+#include "codes/DummyPrefixCodeDecoder.h" // for DummyPrefixCodeDecoder
+#include "codes/PrefixCodeDecoder.h"      // for PrefixCodeDecoder, HuffmanTa...
+#include "common/RawImage.h"              // for RawImage
+#include "common/RawspeedException.h"     // for ThrowException
+#include "decoders/RawDecoderException.h" // for ThrowException, ThrowRDE
+#include "io/ByteStream.h"                // for ByteStream
+#include <array>                          // for array
+#include <cassert>                        // for invariant
+#include <cstddef>                        // for size_t, ptrdiff_t
+#include <cstdint>                        // for uint16_t
+#include <functional>                     // for reference_wrapper
+#include <iterator>                       // for input_iterator_tag
+#include <tuple>                          // for tuple
+#include <utility>                        // for index_sequence
+#include <vector>                         // for vector
 
 namespace rawspeed {
 
 class ByteStream;
 class RawImage;
-
 struct Cr2SliceWidthIterator;
 struct Cr2SliceIterator;
 struct Cr2OutputTileIterator;
@@ -56,7 +57,7 @@ class Cr2SliceWidths {
   friend class Cr2LJpegDecoder;
   friend struct Cr2SliceWidthIterator;
 
-  template <typename HuffmanTable> friend class Cr2Decompressor;
+  template <typename PrefixCodeDecoder> friend class Cr2Decompressor;
 
 public:
   Cr2SliceWidths() = default;
@@ -74,7 +75,7 @@ public:
   }
 
   [[nodiscard]] int widthOfSlice(int sliceId) const {
-    assert(sliceId >= 0 && sliceId < numSlices);
+    invariant(sliceId >= 0 && sliceId < numSlices);
     if ((sliceId + 1) == numSlices)
       return lastSliceWidth;
     return sliceWidth;
@@ -97,11 +98,13 @@ struct Cr2SliceWidthIterator final {
 
   Cr2SliceWidthIterator(const Cr2SliceWidths& slicing_, int sliceId_)
       : slicing(slicing_), sliceId(sliceId_) {
-    assert(sliceId >= 0 && sliceId <= slicing.numSlices && "Iterator overflow");
+    invariant(sliceId >= 0 && sliceId <= slicing.numSlices &&
+              "Iterator overflow");
   }
 
   value_type operator*() const {
-    assert(sliceId >= 0 && sliceId < slicing.numSlices && "Iterator overflow");
+    invariant(sliceId >= 0 && sliceId < slicing.numSlices &&
+              "Iterator overflow");
     return slicing.widthOfSlice(sliceId);
   }
   Cr2SliceWidthIterator& operator++() {
@@ -110,7 +113,7 @@ struct Cr2SliceWidthIterator final {
   }
   friend bool operator==(const Cr2SliceWidthIterator& a,
                          const Cr2SliceWidthIterator& b) {
-    assert(&a.slicing == &b.slicing && "Comparing unrelated iterators.");
+    invariant(&a.slicing == &b.slicing && "Comparing unrelated iterators.");
     return a.sliceId == b.sliceId;
   }
   friend bool operator!=(const Cr2SliceWidthIterator& a,
@@ -126,10 +129,10 @@ inline Cr2SliceWidthIterator Cr2SliceWidths::end() const {
   return {*this, numSlices};
 }
 
-template <typename HuffmanTable> class Cr2Decompressor final {
+template <typename PrefixCodeDecoder> class Cr2Decompressor final {
 public:
   struct PerComponentRecipe {
-    const HuffmanTable& ht;
+    const PrefixCodeDecoder& ht;
     const uint16_t initPred;
   };
 
@@ -145,12 +148,14 @@ private:
   const ByteStream input;
 
   template <int N_COMP, size_t... I>
-  [[nodiscard]] std::array<std::reference_wrapper<const HuffmanTable>, N_COMP>
-      getHuffmanTablesImpl(std::index_sequence<I...> /*unused*/) const;
+  [[nodiscard]] std::array<std::reference_wrapper<const PrefixCodeDecoder>,
+                           N_COMP>
+      getPrefixCodeDecodersImpl(std::index_sequence<I...> /*unused*/) const;
 
   template <int N_COMP>
-  [[nodiscard]] std::array<std::reference_wrapper<const HuffmanTable>, N_COMP>
-  getHuffmanTables() const;
+  [[nodiscard]] std::array<std::reference_wrapper<const PrefixCodeDecoder>,
+                           N_COMP>
+  getPrefixCodeDecoders() const;
 
   template <int N_COMP>
   [[nodiscard]] std::array<uint16_t, N_COMP> getInitialPreds() const;
@@ -173,6 +178,6 @@ public:
   void decompress();
 };
 
-extern template class Cr2Decompressor<HuffmanTable>;
+extern template class Cr2Decompressor<PrefixCodeDecoder<>>;
 
 } // namespace rawspeed
