@@ -20,6 +20,7 @@
 
 #pragma once
 
+#include "codes/PrefixCodeDecoder.h"
 #include "common/RawImage.h"                    // for RawImage
 #include "decompressors/AbstractDecompressor.h" // for AbstractDecompressor
 #include "io/BitPumpMSB.h"                      // for BitPumpMSB
@@ -49,7 +50,7 @@ class NikonDecompressor final : public AbstractDecompressor {
 public:
   NikonDecompressor(const RawImage& raw, ByteStream metadata, uint32_t bitsPS);
 
-  void decompress(const ByteStream& data, bool uncorrectedRawValues);
+  void decompress(ByteStream data, bool uncorrectedRawValues);
 
 private:
   static const std::array<std::array<std::array<uint8_t, 16>, 2>, 6> nikon_tree;
@@ -61,7 +62,12 @@ private:
   void decompress(BitPumpMSB& bits, int start_y, int end_y);
 
   template <typename Huffman>
-  static Huffman createHuffmanTable(uint32_t huffSelect);
+  static Huffman createPrefixCodeDecoder(uint32_t huffSelect);
 };
 
+template <>
+PrefixCodeDecoder<>
+NikonDecompressor::createPrefixCodeDecoder<PrefixCodeDecoder<>>(
+    uint32_t huffSelect);
+
 } // namespace rawspeed
