@@ -22,12 +22,14 @@
 
 #include "decompressors/SamsungV0Decompressor.h"
 #include "adt/Array2DRef.h"               // for Array2DRef
+#include "adt/Invariant.h"                // for invariant
 #include "adt/Point.h"                    // for iPoint2D
 #include "common/Common.h"                // for signExtend
 #include "common/RawImage.h"              // for RawImage, RawImageData
 #include "decoders/RawDecoderException.h" // for ThrowException, ThrowRDE
 #include "io/BitPumpMSB32.h"              // for BitPumpMSB32
 #include "io/ByteStream.h"                // for ByteStream
+#include <algorithm>                      // for max
 #include <array>                          // for array
 #include <cassert>                        // for assert
 #include <cstdint>                        // for uint32_t, uint16_t, int32_t
@@ -38,8 +40,7 @@
 namespace rawspeed {
 
 SamsungV0Decompressor::SamsungV0Decompressor(const RawImage& image,
-                                             const ByteStream& bso,
-                                             const ByteStream& bsr)
+                                             ByteStream bso, ByteStream bsr)
     : AbstractSamsungDecompressor(image) {
   if (mRaw->getCpp() != 1 || mRaw->getDataType() != RawImageType::UINT16 ||
       mRaw->getBpp() != sizeof(uint16_t))
@@ -75,7 +76,7 @@ void SamsungV0Decompressor::computeStripes(ByteStream bso, ByteStream bsr) {
       ThrowRDE("Line offsets are out of sequence or slice is empty.");
 
     const auto size = *next_offset_iterator - *offset_iterator;
-    assert(size > 0);
+    invariant(size > 0);
 
     stripes.emplace_back(bsr.getStream(size));
 
@@ -104,10 +105,9 @@ int32_t SamsungV0Decompressor::calcAdj(BitPumpMSB32& bits, int nbits) {
   return signExtend(bits.getBits(nbits), nbits);
 }
 
-void SamsungV0Decompressor::decompressStrip(int row,
-                                            const ByteStream& bs) const {
+void SamsungV0Decompressor::decompressStrip(int row, ByteStream bs) const {
   const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
-  assert(out.width > 0);
+  invariant(out.width > 0);
 
   BitPumpMSB32 bits(bs);
 
