@@ -22,7 +22,8 @@
 
 #include "decompressors/UncompressedDecompressor.h"
 #include "adt/Array2DRef.h"               // for Array2DRef
-#include "adt/Point.h"                    // for iPoint2D
+#include "adt/Invariant.h"                // for invariant
+#include "adt/Point.h"                    // for iPoint2D, iRectangle2D
 #include "common/Common.h"                // for BitOrder, copyPixels, BitO...
 #include "common/FloatingPoint.h"         // for Binary16, Binary24, Binary...
 #include "decoders/RawDecoderException.h" // for ThrowRDE
@@ -34,7 +35,7 @@
 #include "io/Endianness.h"                // for Endianness, Endianness::li...
 #include "io/IOException.h"               // for ThrowException, ThrowIOE
 #include <algorithm>                      // for min
-#include <cassert>                        // for assert
+#include <cinttypes>                      // for PRIu64
 
 using std::min;
 
@@ -42,10 +43,10 @@ namespace rawspeed {
 
 void UncompressedDecompressor::sanityCheck(const uint32_t* h,
                                            int bytesPerLine) const {
-  assert(h != nullptr);
-  assert(*h > 0);
-  assert(bytesPerLine > 0);
-  assert(input.getSize() > 0);
+  invariant(h != nullptr);
+  invariant(*h > 0);
+  invariant(bytesPerLine > 0);
+  invariant(input.getSize() > 0);
 
   // How many multiples of bpl are there in the input buffer?
   // The remainder is ignored/discarded.
@@ -66,18 +67,18 @@ void UncompressedDecompressor::sanityCheck(const uint32_t* h,
 
 void UncompressedDecompressor::sanityCheck(uint32_t w, const uint32_t* h,
                                            int bpp) const {
-  assert(w > 0);
-  assert(bpp > 0);
+  invariant(w > 0);
+  invariant(bpp > 0);
 
   // bytes per line
   const auto bpl = bpp * w;
-  assert(bpl > 0);
+  invariant(bpl > 0);
 
   sanityCheck(h, bpl);
 }
 
 int UncompressedDecompressor::bytesPerLine(int w, bool skips) {
-  assert(w > 0);
+  invariant(w > 0);
 
   if ((12 * w) % 8 != 0)
     ThrowIOE("Bad image width");
@@ -94,11 +95,61 @@ int UncompressedDecompressor::bytesPerLine(int w, bool skips) {
   return perline;
 }
 
+UncompressedDecompressor::UncompressedDecompressor(
+    ByteStream input_, const RawImage& img_, const iRectangle2D& crop,
+    int inputPitchBytes_, int bitPerPixel_, BitOrder order_)
+    : input(input_.getStream(crop.dim.y, inputPitchBytes_)), mRaw(img_),
+      size(crop.dim), offset(crop.pos), inputPitchBytes(inputPitchBytes_),
+      bitPerPixel(bitPerPixel_), order(order_) {
+  if (!size.hasPositiveArea())
+    ThrowRDE("Empty tile.");
+
+  if (inputPitchBytes < 1)
+    ThrowRDE("Input pitch is non-positive");
+
+  uint32_t w = size.x;
+  uint32_t h = size.y;
+  uint32_t cpp = mRaw->getCpp();
+  uint64_t ox = offset.x;
+  uint64_t oy = offset.y;
+
+  if (cpp < 1 || cpp > 3)
+    ThrowRDE("Unsupported number of components per pixel: %u", cpp);
+
+  if (bitPerPixel < 1 || bitPerPixel > 32 ||
+      (bitPerPixel > 16 && mRaw->getDataType() == RawImageType::UINT16))
+    ThrowRDE("Unsupported bit depth");
+
+  const auto outPixelBits = (uint64_t)w * cpp * bitPerPixel;
+  invariant(outPixelBits > 0);
+
+  if (outPixelBits % 8 != 0) {
+    ThrowRDE("Bad combination of cpp (%u), bps (%u) and width (%u), the "
+             "pitch is %" PRIu64 " bits, which is not a multiple of 8 (1 byte)",
+             cpp, bitPerPixel, w, outPixelBits);
+  }
+
+  const auto outPixelBytes = outPixelBits / 8;
+
+  // The input pitch might be larger than needed (i.e. have some padding),
+  // but it can *not* be smaller than needed.
+  if ((unsigned)inputPitchBytes < outPixelBytes)
+    ThrowRDE("Specified pitch is smaller than minimally-required pitch");
+
+  // Check the specified pitch, not the minimally-required pitch.
+  sanityCheck(&h, inputPitchBytes);
+
+  invariant((unsigned)inputPitchBytes >= outPixelBytes);
+  skipBytes = inputPitchBytes - outPixelBytes; // Skip per line
+
+  if (oy > static_cast<uint64_t>(mRaw->dim.y))
+    ThrowRDE("Invalid y offset");
+  if (ox + size.x > static_cast<uint64_t>(mRaw->dim.x))
+    ThrowRDE("Invalid x offset");
+}
+
 template <typename Pump, typename NarrowFpType>
-void UncompressedDecompressor::decodePackedFP(const iPoint2D& size,
-                                              const iPoint2D& offset,
-                                              uint32_t skipBytes, int rows,
-                                              int row) const {
+void UncompressedDecompressor::decodePackedFP(int rows, int row) const {
   const Array2DRef<float> out(mRaw->getF32DataAsUncroppedArray2DRef());
   Pump bits(input);
 
@@ -115,10 +166,7 @@ void UncompressedDecompressor::decodePackedFP(const iPoint2D& size,
 }
 
 template <typename Pump>
-void UncompressedDecompressor::decodePackedInt(const iPoint2D& size,
-                                               const iPoint2D& offset,
-                                               uint32_t skipBytes, int rows,
-                                               int row, int bitPerPixel) const {
+void UncompressedDecompressor::decodePackedInt(int rows, int row) const {
   const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
   Pump bits(input);
 
@@ -131,51 +179,13 @@ void UncompressedDecompressor::decodePackedInt(const iPoint2D& size,
   }
 }
 
-void UncompressedDecompressor::readUncompressedRaw(const iPoint2D& size,
-                                                   const iPoint2D& offset,
-                                                   int inputPitchBytes,
-                                                   int bitPerPixel,
-                                                   BitOrder order) {
-  assert(inputPitchBytes > 0);
-  assert(bitPerPixel > 0);
-
+void UncompressedDecompressor::readUncompressedRaw() {
   uint32_t outPitch = mRaw->pitch;
   uint32_t w = size.x;
   uint32_t h = size.y;
   uint32_t cpp = mRaw->getCpp();
-  uint64_t ox = offset.x;
   uint64_t oy = offset.y;
 
-  if (bitPerPixel > 16 && mRaw->getDataType() == RawImageType::UINT16)
-    ThrowRDE("Unsupported bit depth");
-
-  const int outPixelBits = w * cpp * bitPerPixel;
-  assert(outPixelBits > 0);
-
-  if (outPixelBits % 8 != 0) {
-    ThrowRDE("Bad combination of cpp (%u), bps (%u) and width (%u), the "
-             "pitch is %u bits, which is not a multiple of 8 (1 byte)",
-             cpp, bitPerPixel, w, outPixelBits);
-  }
-
-  const int outPixelBytes = outPixelBits / 8;
-
-  // The input pitch might be larger than needed (i.e. have some padding),
-  // but it can *not* be smaller than needed.
-  if (inputPitchBytes < outPixelBytes)
-    ThrowRDE("Specified pitch is smaller than minimally-required pitch");
-
-  // Check the specified pitch, not the minimally-required pitch.
-  sanityCheck(&h, inputPitchBytes);
-
-  assert(inputPitchBytes >= outPixelBytes);
-  uint32_t skipBytes = inputPitchBytes - outPixelBytes; // Skip per line
-
-  if (oy > static_cast<uint64_t>(mRaw->dim.y))
-    ThrowRDE("Invalid y offset");
-  if (ox + size.x > static_cast<uint64_t>(mRaw->dim.x))
-    ThrowRDE("Invalid x offset");
-
   uint64_t y = oy;
   h = min(h + oy, static_cast<uint64_t>(mRaw->dim.y));
 
@@ -188,23 +198,19 @@ void UncompressedDecompressor::readUncompressedRaw(const iPoint2D& size,
       return;
     }
     if (BitOrder::MSB == order && bitPerPixel == 16) {
-      decodePackedFP<BitPumpMSB, ieee_754_2008::Binary16>(size, offset,
-                                                          skipBytes, h, y);
+      decodePackedFP<BitPumpMSB, ieee_754_2008::Binary16>(h, y);
       return;
     }
     if (BitOrder::LSB == order && bitPerPixel == 16) {
-      decodePackedFP<BitPumpLSB, ieee_754_2008::Binary16>(size, offset,
-                                                          skipBytes, h, y);
+      decodePackedFP<BitPumpLSB, ieee_754_2008::Binary16>(h, y);
       return;
     }
     if (BitOrder::MSB == order && bitPerPixel == 24) {
-      decodePackedFP<BitPumpMSB, ieee_754_2008::Binary24>(size, offset,
-                                                          skipBytes, h, y);
+      decodePackedFP<BitPumpMSB, ieee_754_2008::Binary24>(h, y);
       return;
     }
     if (BitOrder::LSB == order && bitPerPixel == 24) {
-      decodePackedFP<BitPumpLSB, ieee_754_2008::Binary24>(size, offset,
-                                                          skipBytes, h, y);
+      decodePackedFP<BitPumpLSB, ieee_754_2008::Binary24>(h, y);
       return;
     }
     ThrowRDE("Unsupported floating-point input bitwidth/bit packing: %u / %u",
@@ -212,11 +218,11 @@ void UncompressedDecompressor::readUncompressedRaw(const iPoint2D& size,
   }
 
   if (BitOrder::MSB == order) {
-    decodePackedInt<BitPumpMSB>(size, offset, skipBytes, h, y, bitPerPixel);
+    decodePackedInt<BitPumpMSB>(h, y);
   } else if (BitOrder::MSB16 == order) {
-    decodePackedInt<BitPumpMSB16>(size, offset, skipBytes, h, y, bitPerPixel);
+    decodePackedInt<BitPumpMSB16>(h, y);
   } else if (BitOrder::MSB32 == order) {
-    decodePackedInt<BitPumpMSB32>(size, offset, skipBytes, h, y, bitPerPixel);
+    decodePackedInt<BitPumpMSB32>(h, y);
   } else {
     if (bitPerPixel == 16 && getHostEndianness() == Endianness::little) {
       const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
@@ -225,17 +231,14 @@ void UncompressedDecompressor::readUncompressedRaw(const iPoint2D& size,
                  w * mRaw->getBpp(), h - y);
       return;
     }
-    if (bitPerPixel == 12 && static_cast<int>(w) == inputPitchBytes * 8 / 12 &&
-        getHostEndianness() == Endianness::little) {
-      decode12BitRaw<Endianness::little>(w, h);
-      return;
-    }
-    decodePackedInt<BitPumpLSB>(size, offset, skipBytes, h, y, bitPerPixel);
+    decodePackedInt<BitPumpLSB>(h, y);
   }
 }
 
 template <bool uncorrectedRawValues>
-void UncompressedDecompressor::decode8BitRaw(uint32_t w, uint32_t h) {
+void UncompressedDecompressor::decode8BitRaw() {
+  uint32_t w = size.x;
+  uint32_t h = size.y;
   sanityCheck(w, &h, 1);
 
   const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
@@ -254,13 +257,13 @@ void UncompressedDecompressor::decode8BitRaw(uint32_t w, uint32_t h) {
   }
 }
 
-template void UncompressedDecompressor::decode8BitRaw<false>(uint32_t w,
-                                                             uint32_t h);
-template void UncompressedDecompressor::decode8BitRaw<true>(uint32_t w,
-                                                            uint32_t h);
+template void UncompressedDecompressor::decode8BitRaw<false>();
+template void UncompressedDecompressor::decode8BitRaw<true>();
 
-template <Endianness e, bool interlaced, bool skips>
-void UncompressedDecompressor::decode12BitRaw(uint32_t w, uint32_t h) {
+template <Endianness e>
+void UncompressedDecompressor::decode12BitRawWithControl() {
+  uint32_t w = size.x;
+  uint32_t h = size.y;
   static constexpr const auto bits = 12;
 
   static_assert(e == Endianness::little || e == Endianness::big,
@@ -274,37 +277,26 @@ void UncompressedDecompressor::decode12BitRaw(uint32_t w, uint32_t h) {
 
   static_assert(bits == 12 && mask == 0x0f, "wrong mask");
 
-  uint32_t perline = bytesPerLine(w, skips);
+  uint32_t perline = bytesPerLine(w, /*skips=*/true);
 
   sanityCheck(&h, perline);
 
   const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
 
-  // FIXME: maybe check size of interlaced data?
   const uint8_t* in = input.peekData(perline * h);
-  uint32_t half = (h + 1) >> 1;
   for (uint32_t row = 0; row < h; row++) {
-    uint32_t y = !interlaced ? row : row % half * 2 + row / half;
-
-    if (interlaced && y == 1) {
-      // The second field starts at a 2048 byte alignment
-      const uint32_t offset = ((half * w * 3 / 2 >> 11) + 1) << 11;
-      input.skipBytes(offset);
-      in = input.peekData(perline * (h - row));
-    }
-
     for (uint32_t x = 0; x < w; x += 2) {
       uint32_t g1 = in[0];
       uint32_t g2 = in[1];
 
-      auto process = [out, y](uint32_t i, bool invert, uint32_t p1,
-                              uint32_t p2) {
+      auto process = [out, row](uint32_t i, bool invert, uint32_t p1,
+                                uint32_t p2) {
         uint16_t pix;
         if (!(invert ^ (e == Endianness::little)))
           pix = (p1 << pack) | (p2 >> pack);
         else
           pix = ((p2 & mask) << 8) | p1;
-        out(y, i) = pix;
+        out(row, i) = pix;
       };
 
       process(x, false, g1, g2);
@@ -315,7 +307,7 @@ void UncompressedDecompressor::decode12BitRaw(uint32_t w, uint32_t h) {
 
       in += 3;
 
-      if (skips && ((x % 10) == 8))
+      if ((x % 10) == 8)
         in++;
     }
   }
@@ -323,59 +315,14 @@ void UncompressedDecompressor::decode12BitRaw(uint32_t w, uint32_t h) {
 }
 
 template void
-UncompressedDecompressor::decode12BitRaw<Endianness::little, false, false>(
-    uint32_t w, uint32_t h);
+UncompressedDecompressor::decode12BitRawWithControl<Endianness::little>();
 template void
-UncompressedDecompressor::decode12BitRaw<Endianness::big, false, false>(
-    uint32_t w, uint32_t h);
-template void
-UncompressedDecompressor::decode12BitRaw<Endianness::big, true, false>(
-    uint32_t w, uint32_t h);
-template void
-UncompressedDecompressor::decode12BitRaw<Endianness::little, false, true>(
-    uint32_t w, uint32_t h);
-template void
-UncompressedDecompressor::decode12BitRaw<Endianness::big, false, true>(
-    uint32_t w, uint32_t h);
+UncompressedDecompressor::decode12BitRawWithControl<Endianness::big>();
 
 template <Endianness e>
-void UncompressedDecompressor::decode12BitRawUnpackedLeftAligned(uint32_t w,
-                                                                 uint32_t h) {
-  static_assert(e == Endianness::big, "unknown endianness");
-
-  sanityCheck(w, &h, 2);
-
-  const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
-  const uint8_t* in = input.getData(w * h * 2);
-
-  for (int row = 0; row < (int)h; row++) {
-    for (int col = 0; col < (int)w; col += 1, in += 2) {
-      uint32_t g1 = in[0];
-      uint32_t g2 = in[1];
-
-      if (e == Endianness::big)
-        out(row, col) = (((g1 << 8) | (g2 & 0xf0)) >> 4);
-    }
-  }
-}
-
-template void
-UncompressedDecompressor::decode12BitRawUnpackedLeftAligned<Endianness::big>(
-    uint32_t w, uint32_t h);
-
-template <int bits, Endianness e>
-void UncompressedDecompressor::decodeRawUnpacked(uint32_t w, uint32_t h) {
-  static_assert(bits == 12 || bits == 14 || bits == 16, "unhandled bitdepth");
-  static_assert(e == Endianness::little || e == Endianness::big,
-                "unknown endianness");
-
-  static constexpr const auto shift = 16 - bits;
-  static constexpr const auto mask = (1 << (8 - shift)) - 1;
-
-  static_assert((bits == 12 && mask == 0x0f) || bits != 12, "wrong mask");
-  static_assert((bits == 14 && mask == 0x3f) || bits != 14, "wrong mask");
-  static_assert((bits == 16 && mask == 0xff) || bits != 16, "wrong mask");
-
+void UncompressedDecompressor::decode12BitRawUnpackedLeftAligned() {
+  uint32_t w = size.x;
+  uint32_t h = size.y;
   sanityCheck(w, &h, 2);
 
   const Array2DRef<uint16_t> out(mRaw->getU16DataAsUncroppedArray2DRef());
@@ -388,28 +335,17 @@ void UncompressedDecompressor::decodeRawUnpacked(uint32_t w, uint32_t h) {
 
       uint16_t pix;
       if (e == Endianness::little)
-        pix = ((g2 << 8) | g1) >> shift;
+        pix = (g2 << 8) | g1;
       else
-        pix = ((g1 & mask) << 8) | g2;
-      out(row, col) = pix;
+        pix = (g1 << 8) | g2;
+      out(row, col) = pix >> 4;
     }
   }
 }
 
 template void
-UncompressedDecompressor::decodeRawUnpacked<12, Endianness::little>(uint32_t w,
-                                                                    uint32_t h);
-template void
-UncompressedDecompressor::decodeRawUnpacked<12, Endianness::big>(uint32_t w,
-                                                                 uint32_t h);
-template void
-UncompressedDecompressor::decodeRawUnpacked<14, Endianness::big>(uint32_t w,
-                                                                 uint32_t h);
-template void
-UncompressedDecompressor::decodeRawUnpacked<16, Endianness::little>(uint32_t w,
-                                                                    uint32_t h);
-template void
-UncompressedDecompressor::decodeRawUnpacked<16, Endianness::big>(uint32_t w,
-                                                                 uint32_t h);
+UncompressedDecompressor::decode12BitRawUnpackedLeftAligned<Endianness::big>();
+template void UncompressedDecompressor::decode12BitRawUnpackedLeftAligned<
+    Endianness::little>();
 
 } // namespace rawspeed
