@@ -31,15 +31,15 @@
 #ifdef HAVE_PUGIXML
 #include <pugixml.hpp> // for xml_document, xml_pars...
 
-using pugi::xml_node;
 using pugi::xml_document;
+using pugi::xml_node;
 using pugi::xml_parse_result;
 #endif
 
 namespace rawspeed {
 
 #ifdef HAVE_PUGIXML
-CameraMetaData::CameraMetaData(const char *docname) {
+CameraMetaData::CameraMetaData(const char* docname) {
   xml_document doc;
 
   if (xml_parse_result result =
@@ -91,12 +91,11 @@ const Camera* CameraMetaData::getCamera(const std::string& make,
                                         const std::string& model) const {
   auto id = getId(make, model, "");
 
-  auto iter = find_if(cameras.cbegin(), cameras.cend(),
-                      [&id](decltype(*cameras.cbegin())& i) {
-                        const auto& cid = i.first;
-                        return tie(id.make, id.model) ==
-                               tie(cid.make, cid.model);
-                      });
+  auto iter = find_if(
+      cameras.cbegin(), cameras.cend(), [&id](decltype(*cameras.cbegin())& i) {
+        const auto& cid = i.first;
+        return tie(id.make, id.model) == tie(cid.make, cid.model);
+      });
 
   if (iter == cameras.end())
     return nullptr;
@@ -110,14 +109,13 @@ bool CameraMetaData::hasCamera(const std::string& make,
   return getCamera(make, model, mode);
 }
 
-const Camera* __attribute__((pure))
+const Camera* RAWSPEED_READONLY
 CameraMetaData::getChdkCamera(uint32_t filesize) const {
   auto camera = chdkCameras.find(filesize);
   return camera == chdkCameras.end() ? nullptr : camera->second;
 }
 
-bool __attribute__((pure))
-CameraMetaData::hasChdkCamera(uint32_t filesize) const {
+bool RAWSPEED_READONLY CameraMetaData::hasChdkCamera(uint32_t filesize) const {
   return chdkCameras.end() != chdkCameras.find(filesize);
 }
 
