@@ -20,11 +20,12 @@
 
 #include "EndiannessTest.h"
 #include "io/Endianness.h" // for getHostEndianness, Endianness, getByteSwa...
+#include <algorithm>       // for copy
 #include <cstring>         // for memcpy, memcmp
-#include <gtest/gtest.h>   // for ParamIteratorInterface, Message, TestPart...
-#include <iomanip>         // for setfill, setw, _Setw, _Setfill
+#include <iomanip>         // for operator<<, setfill, setw, _Setw, _Setfill
 #include <iostream>        // for hex, endl, ostream
 #include <memory>          // for allocator
+#include <gtest/gtest.h>   // for ParamIteratorInterface, Message, TestPart...
 
 using rawspeed::Endianness;
 using rawspeed::getBE;
