@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2010-2022 darktable developers.
+    Copyright (C) 2010-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -192,7 +192,8 @@ static void save_usercss(GtkTextBuffer *buffer)
   GError *error = NULL;
   if(!g_file_set_contents(usercsspath, usercsscontent, -1, &error))
   {
-    fprintf(stderr, "%s: error saving css to %s: %s\n", G_STRFUNC, usercsspath, error->message);
+    dt_print(DT_DEBUG_ALWAYS, "%s: error saving css to %s: %s\n",
+             G_STRFUNC, usercsspath, error->message);
     g_clear_error(&error);
   }
   g_free(usercsscontent);
@@ -410,13 +411,18 @@ static void init_tab_general(GtkWidget *dialog, GtkWidget *stack, dt_gui_themetw
   gtk_container_add(GTK_CONTAINER(scroll), tw->css_text_view);
   gtk_box_pack_start(GTK_BOX(usercssbox), scroll, TRUE, TRUE, 0);
 
+  GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   tw->save_button = gtk_button_new_with_label(C_("usercss", "save CSS and apply"));
   g_signal_connect(G_OBJECT(tw->save_button), "clicked", G_CALLBACK(save_usercss_callback), tw);
   g_signal_connect(G_OBJECT(dialog), "response", G_CALLBACK(usercss_dialog_callback), tw);
-  GtkWidget *hbox = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
   gtk_box_pack_end(GTK_BOX(hbox), tw->save_button, FALSE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(usercssbox), hbox, FALSE, FALSE, 0);
   gtk_widget_set_tooltip_text(tw->save_button, _("click to save and apply the CSS tweaks entered in this editor"));
+  GtkWidget *button = gtk_button_new_with_label(_("?"));
+  gtk_widget_set_tooltip_text(button, _("open help page for CSS tweaks"));
+  dt_gui_add_help_link(button, "css_tweaks");
+  g_signal_connect(button, "clicked", G_CALLBACK(dt_gui_show_help), NULL);
+  gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
 
   //set textarea text from file or default
   char usercsspath[PATH_MAX] = { 0 }, configdir[PATH_MAX] = { 0 };
@@ -521,13 +527,13 @@ void dt_gui_preferences_show()
 
   //setup tabs
   init_tab_general(_preferences_dialog, stack, tweak_widgets);
-  init_tab_import(_preferences_dialog, stack);
-  init_tab_lighttable(_preferences_dialog, stack);
-  init_tab_darkroom(_preferences_dialog, stack);
-  init_tab_processing(_preferences_dialog, stack);
-  init_tab_security(_preferences_dialog, stack);
-  init_tab_storage(_preferences_dialog, stack);
-  init_tab_misc(_preferences_dialog, stack);
+  init_tab_import(_preferences_dialog, stack, N_("import"));
+  init_tab_lighttable(_preferences_dialog, stack, N_("lighttable"));
+  init_tab_darkroom(_preferences_dialog, stack, N_("darkroom"));
+  init_tab_processing(_preferences_dialog, stack, N_("processing"));
+  init_tab_security(_preferences_dialog, stack, N_("security"));
+  init_tab_storage(_preferences_dialog, stack, N_("storage"));
+  init_tab_misc(_preferences_dialog, stack, N_("miscellaneous"));
   init_tab_accels(stack);
   init_tab_presets(stack);
 
@@ -867,6 +873,11 @@ static void init_tab_presets(GtkWidget *stack)
   gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, TRUE, 0);
   g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(export_preset), (gpointer)model);
 
+  button = gtk_button_new_with_label(_("?"));
+  dt_gui_add_help_link(button, "presets");
+  g_signal_connect(button, "clicked", G_CALLBACK(dt_gui_show_help), NULL);
+  gtk_box_pack_end(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+
   gtk_box_pack_start(GTK_BOX(container), hbox, FALSE, FALSE, 0);
 
   // Attaching treeview signals
