@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2011-2021 darktable developers.
+    Copyright (C) 2011-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -43,23 +43,32 @@
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
-#if defined(__SSE__)
-#include <xmmintrin.h>
-#endif
 
 DT_MODULE_INTROSPECTION(3, dt_iop_borders_params_t)
 
 // Module constants
-#define DT_IOP_BORDERS_ASPECT_COUNT 12
-#define DT_IOP_BORDERS_ASPECT_IMAGE_IDX 0
-#define DT_IOP_BORDERS_ASPECT_CONSTANT_IDX 11
 #define DT_IOP_BORDERS_ASPECT_IMAGE_VALUE 0.0f
 #define DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE -1.0f
-#define DT_IOP_BORDERS_ASPECT_ORIENTATION_AUTO 0
-#define DT_IOP_BORDERS_ASPECT_ORIENTATION_PORTRAIT 1
-#define DT_IOP_BORDERS_ASPECT_ORIENTATION_LANDSCAPE 2
-#define DT_IOP_BORDERS_POSITION_H_COUNT 5
-#define DT_IOP_BORDERS_POSITION_V_COUNT 5
+typedef enum dt_iop_orientation_t
+{
+  DT_IOP_BORDERS_ASPECT_ORIENTATION_AUTO = 0,      // $DESCRIPTION: "auto"
+  DT_IOP_BORDERS_ASPECT_ORIENTATION_PORTRAIT = 1,  // $DESCRIPTION: "portrait"
+  DT_IOP_BORDERS_ASPECT_ORIENTATION_LANDSCAPE = 2, // $DESCRIPTION: "landscape"
+} dt_iop_orientation_t;
+
+static const float _aspect_ratios[]
+  = { DT_IOP_BORDERS_ASPECT_IMAGE_VALUE,
+      3.0f, 95.0f / 33.0f, 2.39f, 2.0f, 16.0f / 9.0f, 5.0f / 3.0f, 14.0f / 8.5f, PHI, 16.0f / 10.0f,
+      3.0f / 2.0f, 297.0f / 210.0f, M_SQRT2, 7.0f / 5.0f, 4.0f / 3.0f, 11.0f / 8.5f, 14.0f / 11.0f,
+      5.0f / 4.0f, 1.0f, DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE };
+static const float _pos_h_ratios[] = { 0.5f, 1.0f / 3.0f, 3.0f / 8.0f, 5.0f / 8.0f, 2.0f / 3.0f };
+static const float _pos_v_ratios[] = { 0.5f, 1.0f / 3.0f, 3.0f / 8.0f, 5.0f / 8.0f, 2.0f / 3.0f };
+
+#define DT_IOP_BORDERS_ASPECT_COUNT G_N_ELEMENTS(_aspect_ratios)
+#define DT_IOP_BORDERS_ASPECT_IMAGE_IDX 0
+#define DT_IOP_BORDERS_ASPECT_CONSTANT_IDX (DT_IOP_BORDERS_ASPECT_COUNT - 1)
+#define DT_IOP_BORDERS_POSITION_H_COUNT G_N_ELEMENTS(_pos_h_ratios)
+#define DT_IOP_BORDERS_POSITION_V_COUNT G_N_ELEMENTS(_pos_v_ratios)
 
 typedef struct dt_iop_borders_params_t
 {
@@ -68,7 +77,7 @@ typedef struct dt_iop_borders_params_t
                                $MIN: 1.0 $MAX: 3.0 $DEFAULT: DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE $DESCRIPTION: "aspect ratio" */
   char aspect_text[20];     /* aspect ratio of the outer frame w/h (user string version)
                                DEFAULT: "constant border" */
-  int aspect_orient;        /* aspect ratio orientation
+  dt_iop_orientation_t aspect_orient;        /* aspect ratio orientation
                                $DEFAULT: 0 $DESCRIPTION: "orientation" */
   float size;               /* border width relative to overall frame width
                                $MIN: 0.0 $MAX: 0.5 $DEFAULT: 0.1 $DESCRIPTION: "border size" */
@@ -101,9 +110,6 @@ typedef struct dt_iop_borders_gui_data_t
   GtkWidget *pos_v_slider;
   GtkWidget *colorpick;
   GtkWidget *border_picker; // the 1st button
-  float aspect_ratios[DT_IOP_BORDERS_ASPECT_COUNT];
-  float pos_h_ratios[DT_IOP_BORDERS_POSITION_H_COUNT];
-  float pos_v_ratios[DT_IOP_BORDERS_POSITION_V_COUNT];
   GtkWidget *frame_size;
   GtkWidget *frame_offset;
   GtkWidget *frame_colorpick;
@@ -181,6 +187,11 @@ const char *name()
   return _("framing");
 }
 
+const char *aliases()
+{
+  return _("borders|enlarge canvas|expand canvas");
+}
+
 const char **description(struct dt_iop_module_t *self)
 {
   return dt_iop_set_description(self, _("add solid borders or margins around the picture"),
@@ -305,49 +316,57 @@ void modify_roi_out(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t
 
   if(d->aspect == DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE)
   {
-    // for a constant border be sure to base the computation on the larger border, failing that the border
-    // will have a difference size depending on the orientation.
+    // for a constant border be sure to base the computation on the
+    // larger border, failing that the border will have a difference
+    // size depending on the orientation.
 
     if(roi_in->width > roi_in->height || !d->max_border_size)
     {
       // this means: relative to width and constant for height as well:
-      roi_out->width = (float)roi_in->width / (1.0f - size);
+      roi_out->width = roundf((float)roi_in->width / (1.0f - size));
       roi_out->height = roi_in->height + roi_out->width - roi_in->width;
     }
     else
     {
       // this means: relative to height and constant for width as well:
-      roi_out->height = (float)roi_in->height / (1.0f - size);
+      roi_out->height = roundf((float)roi_in->height / (1.0f - size));
       roi_out->width = roi_in->width + roi_out->height - roi_in->height;
     }
   }
   else
   {
-    float image_aspect = roi_in->width / (float)(roi_in->height);
-    float aspect = (d->aspect == DT_IOP_BORDERS_ASPECT_IMAGE_VALUE) ? image_aspect : d->aspect;
+    const float image_aspect = (float)roi_in->width / (float)(roi_in->height);
+
+    float aspect = (d->aspect == DT_IOP_BORDERS_ASPECT_IMAGE_VALUE)
+      ? image_aspect
+      : d->aspect;
 
     if(d->aspect_orient == DT_IOP_BORDERS_ASPECT_ORIENTATION_AUTO)
-      aspect = ((image_aspect < 1 && aspect > 1) || (image_aspect > 1 && aspect < 1)) ? 1 / aspect : aspect;
+      aspect = ((image_aspect < 1.0f && aspect > 1.0f)
+                || (image_aspect > 1.0f && aspect < 1.0f))
+        ? 1.0f / aspect
+        : aspect;
     else if(d->aspect_orient == DT_IOP_BORDERS_ASPECT_ORIENTATION_LANDSCAPE)
-      aspect = (aspect < 1) ? 1 / aspect : aspect;
+      aspect = (aspect < 1.0f) ? 1.0f / aspect : aspect;
     else if(d->aspect_orient == DT_IOP_BORDERS_ASPECT_ORIENTATION_PORTRAIT)
-      aspect = (aspect > 1) ? 1 / aspect : aspect;
+      aspect = (aspect > 1.0f) ? 1.0f / aspect : aspect;
 
     // min width: constant ratio based on size:
-    roi_out->width = (float)roi_in->width / (1.0f - size);
+    roi_out->width = roundf((float)roi_in->width / (1.0f - size));
     // corresponding height: determined by aspect ratio:
-    roi_out->height = (float)roi_out->width / aspect;
+    roi_out->height = roundf((float)roi_out->width / aspect);
     // insane settings used?
     if(roi_out->height < (float)roi_in->height / (1.0f - size))
     {
-      roi_out->height = (float)roi_in->height / (1.0f - size);
-      roi_out->width = (float)roi_out->height * aspect;
+      roi_out->height = roundf((float)roi_in->height / (1.0f - size));
+      roi_out->width = roundf((float)roi_out->height * aspect);
     }
   }
 
   // sanity check.
-  roi_out->width = CLAMP(roi_out->width, 1, 3 * roi_in->width);
-  roi_out->height = CLAMP(roi_out->height, 1, 3 * roi_in->height);
+  const size_t max_dim = MAX(roi_in->width, roi_in->height);
+  roi_out->width = CLAMP(roi_out->width, 1, 3 * max_dim);
+  roi_out->height = CLAMP(roi_out->height, 1, 3 * max_dim);
 }
 
 // 2nd pass: which roi would this operation need as input to fill the given output region?
@@ -356,26 +375,33 @@ void modify_roi_in(struct dt_iop_module_t *self, struct dt_dev_pixelpipe_iop_t *
 {
   dt_iop_borders_data_t *d = (dt_iop_borders_data_t *)piece->data;
   *roi_in = *roi_out;
-  const int bw = (piece->buf_out.width - piece->buf_in.width) * roi_out->scale;
-  const int bh = (piece->buf_out.height - piece->buf_in.height) * roi_out->scale;
+
+  const float bw = (piece->buf_out.width - piece->buf_in.width) * roi_out->scale;
+  const float bh = (piece->buf_out.height - piece->buf_in.height) * roi_out->scale;
 
   // don't request outside image (no px for borders)
-  roi_in->x = MAX(roi_out->x - bw * d->pos_h, 0);
-  roi_in->y = MAX(roi_out->y - bh * d->pos_v, 0);
+  roi_in->x = MAX(roundf(roi_out->x - bw * d->pos_h), 0);
+  roi_in->y = MAX(roundf(roi_out->y - bh * d->pos_v), 0);
+
   // subtract upper left border from dimensions
-  roi_in->width -= MAX(bw * d->pos_h - roi_out->x, 0);
-  roi_in->height -= MAX(bh * d->pos_v - roi_out->y, 0);
+  roi_in->width -= MAX(roundf(bw * d->pos_h - roi_out->x), 0);
+  roi_in->height -= MAX(roundf(bh * d->pos_v - roi_out->y), 0);
 
   // subtract lower right border from dimensions
-  roi_in->width -= roi_out->scale
-                   * MAX((roi_in->x + roi_in->width) / roi_out->scale - (piece->buf_in.width), 0);
-  roi_in->height -= roi_out->scale
-                    * MAX((roi_in->y + roi_in->height) / roi_out->scale - (piece->buf_in.height), 0);
-  // don't request nothing or outside roi
-  roi_in->width = MIN(roi_out->scale * piece->buf_in.width, MAX(1, roi_in->width));
-  roi_in->height = MIN(roi_out->scale * piece->buf_in.height, MAX(1, roi_in->height));
-  // FIXME: clamping to 1 leads to a one-pixel visual glitch if the right/bottom border completely fills the
-  // FIXME: viewport, but changing it to 0 breaks all of the tiling_callback functions with a division by zero
+  const float p_inw = (float)piece->buf_in.width * roi_out->scale;
+  const float p_inh = (float)piece->buf_in.height * roi_out->scale;
+
+  roi_in->width  -= MAX(roundf((float)(roi_in->x + roi_in->width) - p_inw), 0);
+  roi_in->height -= MAX(roundf((float)(roi_in->y + roi_in->height) - p_inh), 0);
+
+  // sanity check: don't request nothing or outside roi
+  roi_in->width = MIN(p_inw, MAX(1, roi_in->width));
+  roi_in->height = MIN(p_inh, MAX(1, roi_in->height));
+
+  // FIXME: clamping to 1 leads to a one-pixel visual glitch if the
+  // right/bottom border completely fills the viewport, but
+  // changing it to 0 breaks all of the tiling_callback functions with
+  // a division by zero.
 }
 
 struct border_positions_t
@@ -404,13 +430,7 @@ static inline void set_pixels(float *buf, const dt_aligned_pixel_t color, const
 {
   for(int i = 0; i < npixels; i++)
   {
-#ifdef _OPENMP
-#pragma omp simd aligned(buf, color : 16)
-#endif
-    for(int c = 0; c < 4; c++)
-    {
-      buf[4*i+c] = color[c];
-    }
+    copy_pixel_nontemporal(buf + 4*i,  color);
   }
 }
 
@@ -419,13 +439,7 @@ static inline void copy_pixels(float *out, const float *const in, const int npix
 {
   for(int i = 0; i < npixels; i++)
   {
-#ifdef _OPENMP
-#pragma omp simd aligned(in, out : 16)
-#endif
-    for(int c = 0; c < 4; c++)
-    {
-      out[4*i+c] = in[4*i+c];
-    }
+    copy_pixel_nontemporal(out + 4*i, in + 4*i);
   }
 }
 
@@ -484,7 +498,8 @@ void copy_image_with_border(float *out, const float *const in, const struct bord
       }
     }
   }
-
+  // ensure that all streaming writes complete before we attempt to read from the output buffer
+  dt_omploop_sfence();
 }
 
 void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
@@ -775,7 +790,7 @@ static void aspect_changed(GtkWidget *combo, dt_iop_module_t *self)
   else if(which < DT_IOP_BORDERS_ASPECT_COUNT)
   {
     g_strlcpy(p->aspect_text, text, sizeof(p->aspect_text));
-    p->aspect = g->aspect_ratios[which];
+    p->aspect = _aspect_ratios[which];
     ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->aspect_slider,p->aspect);
     --darktable.gui->reset;
@@ -797,7 +812,7 @@ static void position_h_changed(GtkWidget *combo, dt_iop_module_t *self)
   else if(which < DT_IOP_BORDERS_POSITION_H_COUNT)
   {
     g_strlcpy(p->pos_h_text, text, sizeof(p->pos_h_text));
-    p->pos_h = g->pos_h_ratios[which];
+    p->pos_h = _pos_h_ratios[which];
     ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->pos_h_slider,p->pos_h);
     --darktable.gui->reset;
@@ -819,7 +834,7 @@ static void position_v_changed(GtkWidget *combo, dt_iop_module_t *self)
   else if(which < DT_IOP_BORDERS_POSITION_V_COUNT)
   {
     g_strlcpy(p->pos_v_text, text, sizeof(p->pos_v_text));
-    p->pos_v = g->pos_v_ratios[which];
+    p->pos_v = _pos_v_ratios[which];
     ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->pos_v_slider,p->pos_v);
     --darktable.gui->reset;
@@ -831,18 +846,35 @@ static void position_v_changed(GtkWidget *combo, dt_iop_module_t *self)
 void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
 {
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
+  dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
-  if(w == g->aspect_slider)
+  int k;
+  if(!w || w == g->aspect_slider)
   {
-    dt_bauhaus_combobox_set(g->aspect, DT_IOP_BORDERS_ASPECT_COUNT);
+    for(k = 0; k < DT_IOP_BORDERS_ASPECT_COUNT; k++)
+    {
+      if(fabsf(p->aspect - _aspect_ratios[k]) < 0.01f)
+        break;
+    }
+    dt_bauhaus_combobox_set(g->aspect, k);
   }
-  else if(w == g->pos_h_slider)
+  else if(!w || w == g->pos_h_slider)
   {
-    dt_bauhaus_combobox_set(g->pos_h, DT_IOP_BORDERS_POSITION_H_COUNT);
+    for(k = 0; k < DT_IOP_BORDERS_POSITION_H_COUNT; k++)
+    {
+      if(fabsf(p->pos_h - _pos_h_ratios[k]) < 0.01f)
+        break;
+    }
+    dt_bauhaus_combobox_set(g->pos_h, k);
   }
-  else if(w == g->pos_v_slider)
+  else if(!w || w == g->pos_v_slider)
   {
-    dt_bauhaus_combobox_set(g->pos_v, DT_IOP_BORDERS_POSITION_V_COUNT);
+    for(k = 0; k < DT_IOP_BORDERS_POSITION_V_COUNT; k++)
+    {
+      if(fabsf(p->pos_v - _pos_v_ratios[k]) < 0.01f)
+        break;
+    }
+    dt_bauhaus_combobox_set(g->pos_v, k);
   }
 }
 
@@ -886,49 +918,7 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
   dt_iop_borders_params_t *p = (dt_iop_borders_params_t *)self->params;
 
-// FIXME by hand
-  // ----- Aspect
-  int k = 0;
-  for(; k < DT_IOP_BORDERS_ASPECT_COUNT; k++)
-  {
-    if(fabsf(p->aspect - g->aspect_ratios[k]) < 0.01f)
-    {
-      dt_bauhaus_combobox_set(g->aspect, k);
-      break;
-    }
-  }
-  if(k == DT_IOP_BORDERS_ASPECT_COUNT)
-  {
-      dt_bauhaus_combobox_set(g->aspect, k);
-  }
-
-  // ----- Position H
-  for(k = 0; k < DT_IOP_BORDERS_POSITION_H_COUNT; k++)
-  {
-    if(fabsf(p->pos_h - g->pos_h_ratios[k]) < 0.01f)
-    {
-      dt_bauhaus_combobox_set(g->pos_h, k);
-      break;
-    }
-  }
-  if(k == DT_IOP_BORDERS_POSITION_H_COUNT)
-  {
-    dt_bauhaus_combobox_set(g->pos_h, k);
-  }
-
-  // ----- Position V
-  for(k = 0; k < DT_IOP_BORDERS_POSITION_V_COUNT; k++)
-  {
-    if(fabsf(p->pos_v - g->pos_v_ratios[k]) < 0.01f)
-    {
-      dt_bauhaus_combobox_set(g->pos_v, k);
-      break;
-    }
-  }
-  if(k == DT_IOP_BORDERS_POSITION_V_COUNT)
-  {
-    dt_bauhaus_combobox_set(g->pos_v, k);
-  }
+  gui_changed(self, NULL, NULL);
 
   // ----- Border Color
   GdkRGBA c = (GdkRGBA){.red = p->color[0], .green = p->color[1], .blue = p->color[2], .alpha = 1.0 };
@@ -941,70 +931,6 @@ void gui_update(struct dt_iop_module_t *self)
   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(g->frame_colorpick), &fc);
 }
 
-static void gui_init_aspect(struct dt_iop_module_t *self)
-{
-  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-
-  dt_bauhaus_combobox_add(g->aspect, _("image"));
-  dt_bauhaus_combobox_add(g->aspect, _("3:1"));
-  dt_bauhaus_combobox_add(g->aspect, _("95:33"));
-  dt_bauhaus_combobox_add(g->aspect, _("2:1"));
-  dt_bauhaus_combobox_add(g->aspect, _("16:9"));
-  dt_bauhaus_combobox_add(g->aspect, _("golden cut"));
-  dt_bauhaus_combobox_add(g->aspect, _("3:2"));
-  dt_bauhaus_combobox_add(g->aspect, _("A4"));
-  dt_bauhaus_combobox_add(g->aspect, _("DIN"));
-  dt_bauhaus_combobox_add(g->aspect, _("4:3"));
-  dt_bauhaus_combobox_add(g->aspect, _("square"));
-  dt_bauhaus_combobox_add(g->aspect, _("constant border"));
-  dt_bauhaus_combobox_add(g->aspect, _("custom..."));
-
-  g->aspect_ratios[DT_IOP_BORDERS_ASPECT_IMAGE_IDX] = DT_IOP_BORDERS_ASPECT_IMAGE_VALUE;
-  g->aspect_ratios[DT_IOP_BORDERS_ASPECT_CONSTANT_IDX] = DT_IOP_BORDERS_ASPECT_CONSTANT_VALUE;
-  int i = 1;
-  g->aspect_ratios[i++] = 3.0f;
-  g->aspect_ratios[i++] = 95.0f / 33.0f;
-  g->aspect_ratios[i++] = 2.0f;
-  g->aspect_ratios[i++] = 16.0f / 9.0f;
-  g->aspect_ratios[i++] = PHI;
-  g->aspect_ratios[i++] = 3.0f / 2.0f;
-  g->aspect_ratios[i++] = 297.0f / 210.0f;
-  g->aspect_ratios[i++] = sqrtf(2.0f);
-  g->aspect_ratios[i++] = 4.0f / 3.0f;
-  g->aspect_ratios[i++] = 1.0f;
-}
-
-static void gui_init_positions(struct dt_iop_module_t *self)
-{
-  dt_iop_borders_gui_data_t *g = (dt_iop_borders_gui_data_t *)self->gui_data;
-
-  dt_bauhaus_combobox_add(g->pos_h, _("center"));
-  dt_bauhaus_combobox_add(g->pos_h, _("1/3"));
-  dt_bauhaus_combobox_add(g->pos_h, _("3/8"));
-  dt_bauhaus_combobox_add(g->pos_h, _("5/8"));
-  dt_bauhaus_combobox_add(g->pos_h, _("2/3"));
-  dt_bauhaus_combobox_add(g->pos_h, _("custom..."));
-  dt_bauhaus_combobox_add(g->pos_v, _("center"));
-  dt_bauhaus_combobox_add(g->pos_v, _("1/3"));
-  dt_bauhaus_combobox_add(g->pos_v, _("3/8"));
-  dt_bauhaus_combobox_add(g->pos_v, _("5/8"));
-  dt_bauhaus_combobox_add(g->pos_v, _("2/3"));
-  dt_bauhaus_combobox_add(g->pos_v, _("custom..."));
-
-  int i = 0;
-  g->pos_h_ratios[i++] = 0.5f;
-  g->pos_h_ratios[i++] = 1.0f / 3.0f;
-  g->pos_h_ratios[i++] = 3.0f / 8.0f;
-  g->pos_h_ratios[i++] = 5.0f / 8.0f;
-  g->pos_h_ratios[i++] = 2.0f / 3.0f;
-  i = 0;
-  g->pos_v_ratios[i++] = 0.5f;
-  g->pos_v_ratios[i++] = 1.0f / 3.0f;
-  g->pos_v_ratios[i++] = 3.0f / 8.0f;
-  g->pos_v_ratios[i++] = 5.0f / 8.0f;
-  g->pos_v_ratios[i++] = 2.0f / 3.0f;
-}
-
 void gui_init(struct dt_iop_module_t *self)
 {
   dt_iop_borders_gui_data_t *g = IOP_GUI_ALLOC(borders);
@@ -1015,44 +941,61 @@ void gui_init(struct dt_iop_module_t *self)
   dt_bauhaus_slider_set_format(g->size, "%");
   gtk_widget_set_tooltip_text(g->size, _("size of the border in percent of the full image"));
 
-  g->aspect = dt_bauhaus_combobox_new(self);
-  dt_bauhaus_combobox_set_editable(g->aspect, 1);
-  dt_bauhaus_widget_set_label(g->aspect, NULL, N_("aspect"));
+  DT_BAUHAUS_COMBOBOX_NEW_FULL(g->aspect, self, NULL, N_("aspect"),
+                               _("select the aspect ratio (right click on slider below to type your own w:h)"),
+                               0, aspect_changed, self,
+                               N_("image"),
+                               N_("3:1"),
+                               N_("95:33"),
+                               N_("CinemaScope 2.39:1"),
+                               N_("2:1"),
+                               N_("16:9"),
+                               N_("5:3"),
+                               N_("US Legal 8.5x14"),
+                               N_("golden cut"),
+                               N_("16:10"),
+                               N_("3:2 (4x6, 10x15cm)"),
+                               N_("A4"),
+                               N_("DIN"),
+                               N_("7:5"),
+                               N_("4:3"),
+                               N_("US Letter 8.5x11"),
+                               N_("14:11"),
+                               N_("5:4 (8x10)"),
+                               N_("square"),
+                               N_("constant border"),
+                               N_("custom..."));
+  dt_bauhaus_combobox_set_editable(g->aspect, 0);
   gtk_box_pack_start(GTK_BOX(self->widget), g->aspect, TRUE, TRUE, 0);
-  gui_init_aspect(self);
-  g_signal_connect(G_OBJECT(g->aspect), "value-changed", G_CALLBACK(aspect_changed), self);
-  gtk_widget_set_tooltip_text(g->aspect, _("select the aspect ratio or right click and type your own (w:h)"));
+
   g->aspect_slider = dt_bauhaus_slider_from_params(self, "aspect");
-  gtk_widget_set_tooltip_text(g->aspect_slider, _("set the custom aspect ratio"));
+  gtk_widget_set_tooltip_text(g->aspect_slider, _("set the custom aspect ratio (right click to enter number or w:h)"));
 
   g->aspect_orient = dt_bauhaus_combobox_from_params(self, "aspect_orient");
-  dt_bauhaus_combobox_add(g->aspect_orient, _("auto"));
-  dt_bauhaus_combobox_add(g->aspect_orient, _("portrait"));
-  dt_bauhaus_combobox_add(g->aspect_orient, _("landscape"));
   gtk_widget_set_tooltip_text(g->aspect_orient, _("aspect ratio orientation of the image with border"));
 
-  g->pos_h = dt_bauhaus_combobox_new(self);
+  DT_BAUHAUS_COMBOBOX_NEW_FULL(g->pos_h, self, NULL, N_("horizontal position"),
+                               _("select the horizontal position ratio relative to top "
+                                 "or right click and type your own (y:h)"),
+                               0, position_h_changed, self,
+                               N_("center"), N_("1/3"), N_("3/8"), N_("5/8"), N_("2/3"), N_("custom..."));
   dt_bauhaus_combobox_set_editable(g->pos_h, 1);
-  dt_bauhaus_widget_set_label(g->pos_h, NULL, N_("horizontal position"));
   gtk_box_pack_start(GTK_BOX(self->widget), g->pos_h, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->pos_h), "value-changed", G_CALLBACK(position_h_changed), self);
-  gtk_widget_set_tooltip_text(g->pos_h, _("select the horizontal position ratio relative to top "
-                                          "or right click and type your own (y:h)"));
+
   g->pos_h_slider = dt_bauhaus_slider_from_params(self, "pos_h");
   gtk_widget_set_tooltip_text(g->pos_h_slider, _("custom horizontal position"));
 
-  g->pos_v = dt_bauhaus_combobox_new(self);
+  DT_BAUHAUS_COMBOBOX_NEW_FULL(g->pos_v, self, NULL, N_("vertical position"),
+                               _("select the vertical position ratio relative to left "
+                                 "or right click and type your own (x:w)"),
+                               0, position_v_changed, self,
+                               N_("center"), N_("1/3"), N_("3/8"), N_("5/8"), N_("2/3"), N_("custom..."));
   dt_bauhaus_combobox_set_editable(g->pos_v, 1);
-  dt_bauhaus_widget_set_label(g->pos_v, NULL, N_("vertical position"));
   gtk_box_pack_start(GTK_BOX(self->widget), g->pos_v, TRUE, TRUE, 0);
-  g_signal_connect(G_OBJECT(g->pos_v), "value-changed", G_CALLBACK(position_v_changed), self);
-  gtk_widget_set_tooltip_text(g->pos_v, _("select the vertical position ratio relative to left "
-                                          "or right click and type your own (x:w)"));
+
   g->pos_v_slider = dt_bauhaus_slider_from_params(self, "pos_v");
   gtk_widget_set_tooltip_text(g->pos_v_slider, _("custom vertical position"));
 
-  gui_init_positions(self);
-
   g->frame_size = dt_bauhaus_slider_from_params(self, "frame_size");
   dt_bauhaus_slider_set_digits(g->frame_size, 4);
   dt_bauhaus_slider_set_format(g->frame_size, "%");
@@ -1077,6 +1020,7 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->colorpick), FALSE, TRUE, 0);
   g->border_picker = dt_color_picker_new(self, DT_COLOR_PICKER_POINT, box);
   gtk_widget_set_tooltip_text(GTK_WIDGET(g->border_picker), _("pick border color from image"));
+  dt_action_define_iop(self, N_("pickers"), N_("border color"), g->border_picker, &dt_action_def_toggle);
   gtk_box_pack_start(GTK_BOX(self->widget), box, TRUE, TRUE, 0);
 
   box = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 0);
@@ -1089,6 +1033,7 @@ void gui_init(struct dt_iop_module_t *self)
   gtk_box_pack_start(GTK_BOX(box), GTK_WIDGET(g->frame_colorpick), FALSE, TRUE, 0);
   g->frame_picker = dt_color_picker_new(self, DT_COLOR_PICKER_POINT, box);
   gtk_widget_set_tooltip_text(GTK_WIDGET(g->frame_picker), _("pick frame line color from image"));
+  dt_action_define_iop(self, N_("pickers"), N_("frame line color"), g->frame_picker, &dt_action_def_toggle);
   gtk_box_pack_start(GTK_BOX(self->widget), box, TRUE, TRUE, 0);
 }
 
@@ -1109,4 +1054,3 @@ void init(dt_iop_module_t *self)
 // vim: shiftwidth=2 expandtab tabstop=2 cindent
 // kate: tab-indents: off; indent-width 2; replace-tabs on; indent-mode cstyle; remove-trailing-spaces modified;
 // clang-format on
-
