@@ -41,10 +41,6 @@
 #include "gui/presets.h"
 #include "iop/iop_api.h"
 
-#if defined(__SSE__)
-#include <xmmintrin.h>
-#endif
-
 DT_MODULE_INTROSPECTION(1, dt_iop_graduatednd_params_t)
 
 typedef struct dt_iop_graduatednd_params_t
@@ -181,7 +177,13 @@ typedef struct dt_iop_vector_2d_t
 } dt_iop_vector_2d_t;
 
 // determine the distance between the segment [(xa,ya)(xb,yb)] and the point (xc,yc)
-static float dist_seg(float xa, float ya, float xb, float yb, float xc, float yc)
+static float _dist_seg(
+  	float xa,
+        float ya,
+        float xb,
+        float yb,
+        float xc,
+        float yc)
 {
   if(xa == xb && ya == yb) return (xc - xa) * (xc - xa) + (yc - ya) * (yc - ya);
 
@@ -202,8 +204,14 @@ static float dist_seg(float xa, float ya, float xb, float yb, float xc, float yc
   return un2 - ah2;
 }
 
-static int set_grad_from_points(struct dt_iop_module_t *self, float xa, float ya, float xb, float yb,
-                                float *rotation, float *offset)
+static int _set_grad_from_points(
+	struct dt_iop_module_t *self,
+        float xa,
+        float ya,
+        float xb,
+        float yb,
+        float *rotation,
+        float *offset)
 {
   // we want absolute positions
   float pts[4]
@@ -296,8 +304,14 @@ static int set_grad_from_points(struct dt_iop_module_t *self, float xa, float ya
   return 1;
 }
 
-static int set_points_from_grad(struct dt_iop_module_t *self, float *xa, float *ya, float *xb, float *yb,
-                                float rotation, float offset)
+static int _set_points_from_grad(
+	struct dt_iop_module_t *self,
+        float *xa,
+        float *ya,
+        float *xb,
+        float *yb,
+        float rotation,
+        float offset)
 {
   // we get the extremities of the line
   const float v = (-rotation / 180) * M_PI;
@@ -427,14 +441,17 @@ static int set_points_from_grad(struct dt_iop_module_t *self, float *xa, float *
   return 1;
 }
 
-static inline void update_saturation_slider_end_color(GtkWidget *slider, float hue)
+static inline void _update_saturation_slider_end_color(GtkWidget *slider, float hue)
 {
   dt_aligned_pixel_t rgb;
   hsl2rgb(rgb, hue, 1.0, 0.5);
   dt_bauhaus_slider_set_stop(slider, 1.0, rgb[0], rgb[1], rgb[2]);
 }
 
-void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpipe_iop_t *piece)
+void color_picker_apply(
+	dt_iop_module_t *self,
+        GtkWidget *picker,
+        dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
@@ -455,7 +472,7 @@ void color_picker_apply(dt_iop_module_t *self, GtkWidget *picker, dt_dev_pixelpi
   ++darktable.gui->reset;
   dt_bauhaus_slider_set(g->hue, p->hue);
   dt_bauhaus_slider_set(g->saturation, p->saturation);
-  update_saturation_slider_end_color(g->saturation, p->hue);
+  _update_saturation_slider_end_color(g->saturation, p->hue);
   --darktable.gui->reset;
 
   dt_dev_add_history_item(darktable.develop, self, TRUE);
@@ -466,8 +483,13 @@ void gui_reset(struct dt_iop_module_t *self)
   dt_iop_color_picker_reset(self, TRUE);
 }
 
-void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, int32_t height,
-                     int32_t pointerx, int32_t pointery)
+void gui_post_expose(
+	struct dt_iop_module_t *self,
+        cairo_t *cr,
+        int32_t width,
+        int32_t height,
+        int32_t pointerx,
+        int32_t pointery)
 {
   dt_develop_t *dev = self->dev;
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
@@ -488,7 +510,7 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   // we get the extremities of the line
   if(g->define == 0)
   {
-    if(!set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset)) return;
+    if(!_set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset)) return;
     g->define = 1;
   }
 
@@ -564,7 +586,12 @@ void gui_post_expose(struct dt_iop_module_t *self, cairo_t *cr, int32_t width, i
   cairo_stroke(cr);
 }
 
-int mouse_moved(struct dt_iop_module_t *self, double x, double y, double pressure, int which)
+int mouse_moved(
+	struct dt_iop_module_t *self,
+        double x,
+        double y,
+        double pressure,
+        int which)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   const dt_dev_zoom_t zoom = dt_control_get_dev_zoom();
@@ -615,7 +642,7 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, double pressur
     {
       g->selected = 2;
     }
-    else if(dist_seg(g->xa, g->ya, g->xb, g->yb, pzx, pzy) < ext * ext * 0.5)
+    else if(_dist_seg(g->xa, g->ya, g->xb, g->yb, pzx, pzy) < ext * ext * 0.5)
       g->selected = 3;
   }
 
@@ -623,8 +650,14 @@ int mouse_moved(struct dt_iop_module_t *self, double x, double y, double pressur
   return 1;
 }
 
-int button_pressed(struct dt_iop_module_t *self, double x, double y, double pressure, int which, int type,
-                   uint32_t state)
+int button_pressed(
+	struct dt_iop_module_t *self,
+        double x,
+        double y,
+        double pressure,
+        int which,
+        int type,
+        uint32_t state)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   float pzx, pzy;
@@ -655,7 +688,12 @@ int button_pressed(struct dt_iop_module_t *self, double x, double y, double pres
   return 0;
 }
 
-int button_released(struct dt_iop_module_t *self, double x, double y, int which, uint32_t state)
+int button_released(
+	struct dt_iop_module_t *self,
+        double x,
+        double y,
+        int which,
+        uint32_t state)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
@@ -667,7 +705,7 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
     pzy += 0.5f;
 
     float r = 0.0, o = 0.0;
-    set_grad_from_points(self, g->xa, g->ya, g->xb, g->yb, &r, &o);
+    _set_grad_from_points(self, g->xa, g->ya, g->xb, g->yb, &r, &o);
 
     // if this is a "line dragging, we reset extremities, to be sure they are not outside the image
     if(g->dragging == 3)
@@ -678,7 +716,7 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
        */
 
       r = p->rotation;
-      set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, r, o);
+      _set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, r, o);
     }
     ++darktable.gui->reset;
     dt_bauhaus_slider_set(g->rotation, r);
@@ -693,7 +731,12 @@ int button_released(struct dt_iop_module_t *self, double x, double y, int which,
   return 0;
 }
 
-int scrolled(dt_iop_module_t *self, double x, double y, int up, uint32_t state)
+int scrolled(
+	dt_iop_module_t *self,
+        double x,
+        double y,
+        int up,
+        uint32_t state)
 {
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)self->params;
@@ -729,44 +772,50 @@ int scrolled(dt_iop_module_t *self, double x, double y, int up, uint32_t state)
 #ifdef _OPENMP
 #pragma omp declare simd simdlen(4)
 #endif
-static inline float density_times_length(const float dens, const float length)
+static inline float _density_times_length(const float dens, const float length)
 {
-//  return (dens * CLIP(0.5f + length) / 8.0f);
   return (dens * CLAMP(0.5f + length, 0.0f, 1.0f) / 8.0f);
 }
 
 #ifdef _OPENMP
 #pragma omp declare simd simdlen(4)
 #endif
-static inline float compute_density(const float dens, const float length)
+static inline float _compute_density(const float dens, const float length)
 {
 #if 1
   // !!! approximation is ok only when highest density is 8
   // for input x = (data->density * CLIP( 0.5+length ), calculate 2^x as (e^(ln2*x/8))^8
   // use exp2f approximation to calculate e^(ln2*x/8)
   // in worst case - density==8,CLIP(0.5-length) == 1.0 it gives 0.6% of error
-  const float t = DT_M_LN2f * density_times_length(dens,length);
+  const float t = DT_M_LN2f * _density_times_length(dens,length);
   const float d1 = t * t * 0.5f;
   const float d2 = d1 * t * 0.333333333f;
   const float d3 = d2 * t * 0.25f;
   const float d = 1 + t + d1 + d2 + d3; /* taylor series for e^x till x^4 */
-  // printf("%d %d  %f\n",y,x,d);
   float density = d * d;
   density = density * density;
   density = density * density;
 #else
   // use fair exp2f
+  // for GCC10 on recent hardware, exp2f is actually faster than the above approximation,
+  // but it does not vectorize so it is slower overall
   const float density = exp2f(dens * CLIP(0.5f + length));
 #endif
   return density;
 }
 
-void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
-             void *const ovoid, const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+void process(struct dt_iop_module_t *self,
+             dt_dev_pixelpipe_iop_t *piece,
+             const void *const ivoid,
+             void *const ovoid,
+             const dt_iop_roi_t *const roi_in,
+             const dt_iop_roi_t *const roi_out)
 {
-  const dt_iop_graduatednd_data_t *const data = (const dt_iop_graduatednd_data_t *const)piece->data;
-  const int ch = piece->colors;
+  if(!dt_iop_have_required_input_format(4 /*we need full-color pixels*/, self, piece->colors,
+                                         ivoid, ovoid, roi_in, roi_out))
+    return;	// input buffer has been copied to output unchanged and the trouble flag set
 
+  const dt_iop_graduatednd_data_t *const data = (const dt_iop_graduatednd_data_t *const)piece->data;
   const int ix = (roi_in->x);
   const int iy = (roi_in->y);
   const float iw = piece->buf_in.width * roi_out->scale;
@@ -778,181 +827,149 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   const float v = (-data->rotation / 180) * M_PI;
   const float sinv = sinf(v);
   const float cosv = cosf(v);
+  const float cosv_hh_inv = cosv * hh_inv;
   const float filter_radie = sqrtf((hh * hh) + (hw * hw)) / hh;
   const float offset = data->offset / 100.0f * 2;
 
-  const float filter_hardness = 1.0 / filter_radie / (1.0 - (0.5 + (data->hardness / 100.0) * 0.9 / 2.0)) * 0.5;
+  const float filter_hardness = (1.0f / filter_radie)
+                                / (1.0f - (0.5f + (data->hardness / 100.0f) * 0.9f / 2.0f)) * 0.5f;
 
   const int width = roi_out->width;
   const int height = roi_out->height;
-  if(data->density > 0)
+  const float length_base = sinv * (-1.0f + ix * hw_inv) + cosv - 1.0f + offset;
+  const float length_inc = sinv * hw_inv * filter_hardness;
+  const float density = data->density;
+  // preload data into aligned storage; the compiler will optimize
+  // these into registers when it vectorizes
+  const dt_aligned_pixel_t color = { data->color[0], data->color[1], data->color[2], data->color[3] };
+  const dt_aligned_pixel_t color1 = { data->color1[0], data->color1[1], data->color1[2], data->color1[3] };
+  const dt_aligned_pixel_t zero = { 0.0f, 0.0f, 0.0f, 0.0f };
+
+  if(density > 0)
   {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-    dt_omp_firstprivate(ch, cosv, data, filter_hardness, hh_inv, hw_inv, \
-                        ivoid, ix, iy, offset, ovoid, height, width, sinv) \
+    dt_omp_firstprivate(density, color, color1, zero, filter_hardness, cosv_hh_inv, \
+                        ivoid, ix, iy, offset, ovoid, height, width, sinv, length_inc, \
+                        length_base)  \
     schedule(static)
 #endif
     for(int y = 0; y < height; y++)
     {
-      const size_t k = (size_t)width * y * ch;
+      const size_t k = (size_t)4 * width * y;
       const float *const restrict in = (float *)ivoid + k;
       float *const restrict out = (float *)ovoid + k;
 
-      float length = (sinv * (-1.0 + ix * hw_inv) - cosv * (-1.0 + (iy + y) * hh_inv) - 1.0 + offset)
-                     * filter_hardness;
-      const float length_inc = sinv * hw_inv * filter_hardness;
+      float length = (length_base - (iy + y) * cosv_hh_inv) * filter_hardness;
+      const dt_aligned_pixel_t counts = { 0.0f, 1.0f, 2.0f, 3.0f };
 
-      for(int x = 0; x < width; x++)
+      // process pixels four at a time so that the compiler can
+      // vectorize the density computation
+      for(int x = 0; x+3 < width; x += 4)
       {
-        const float density = compute_density(data->density, length);
-
-        #ifdef _OPENMP
-        #pragma omp simd aligned(in, out : 16)
-        #endif
-        for(int l = 0; l < 4; l++)
+        dt_aligned_pixel_t curr_density;
+        dt_aligned_pixel_t lengths;
+        for_four_channels(i)
         {
-          out[ch*x+l] = MAX(0.0f, (in[ch*x+l] / (data->color[l] + data->color1[l] * density)));
+          lengths[i] = length + counts[i] * length_inc;
+          curr_density[i] = _compute_density(density, lengths[i]);
         }
+        for(int i = 0; i < 4; i++)
+        {
+          dt_aligned_pixel_t res;	// the compiler will optimize this into a register
+          for_each_channel(l, aligned(in : 16))
+          {
+            res[l] = MAX(zero[l], (in[4*(x+i)+l] / (color[l] + color1[l] * curr_density[i])));
+          }
+          // use streaming writes to eliminate the memory reads from loading cache lines
+          copy_pixel_nontemporal(out + 4*(x+i), res);
+        }
+        length += 4 * length_inc;
+      }
+      // handle the left-over pixels
+      for(int x = width & ~3; x < width; x++)
+      {
+        const float curr_density = _compute_density(density, length);
+        dt_aligned_pixel_t res;	// the compiler will optimize this into a register
+        for_each_channel(l, aligned(in : 16))
+        {
+          res[l] = MAX(zero[l], (in[4*x+l] / (color[l] + color1[l] * curr_density)));
+        }
+        // use streaming writes to eliminate the memory reads from loading cache lines
+        copy_pixel_nontemporal(out + 4*x, res);
         length += length_inc;
       }
     }
+    // ensure that the nontemporal writes have finished before continuing
+    dt_omploop_sfence();
   }
   else
   {
 #ifdef _OPENMP
 #pragma omp parallel for default(none) \
-    dt_omp_firstprivate(ch, cosv, data, filter_hardness, hh_inv, hw_inv, \
-                        ivoid, ix, iy, offset, ovoid, height, width, sinv)    \
+    dt_omp_firstprivate(density, color, color1, zero, filter_hardness, cosv_hh_inv,    \
+                        ivoid, ix, iy, offset, ovoid, height, width, sinv, length_inc, length_base) \
     schedule(static)
 #endif
     for(int y = 0; y < height; y++)
     {
-      const size_t k = (size_t)width * y * ch;
+      const size_t k = (size_t)4 * width * y;
       const float *const restrict in = (float *)ivoid + k;
       float *const restrict out = (float *)ovoid + k;
+      float length = (length_base - (iy + y) * cosv_hh_inv) * filter_hardness;
+      const dt_aligned_pixel_t counts = { 0.0f, 1.0f, 2.0f, 3.0f };
 
-      float length = (sinv * (-1.0f + ix * hw_inv) - cosv * (-1.0f + (iy + y) * hh_inv) - 1.0f + offset)
-                     * filter_hardness;
-      const float length_inc = sinv * hw_inv * filter_hardness;
-
-      for(int x = 0; x < width; x++)
+      // process pixels four at a time so that the compiler can
+      // vectorize the density computation
+      for(int x = 0; x+3 < width; x += 4)
       {
-        const float density = compute_density(-data->density, -length);
-
-        #ifdef _OPENMP
-        #pragma omp simd aligned(in, out : 16)
-        #endif
-        for(int l = 0; l < 4; l++)
+        dt_aligned_pixel_t curr_density;
+        dt_aligned_pixel_t lengths;
+        for_four_channels(i)
         {
-          out[ch*x+l] = MAX(0.0f, (in[ch*x+l] * (data->color[l] + data->color1[l] * density)));
+          lengths[i] = -length + counts[i] * length_inc;
+          curr_density[i] = _compute_density(-density, lengths[i]);
         }
+        for(int i = 0; i < 4; i++)
+        {
+          dt_aligned_pixel_t res;	// the compiler will optimize this into a register
+          for_each_channel(l, aligned(in : 16))
+          {
+            res[l] = MAX(zero[l], (in[4*(x+i)+l] * (color[l] + color1[l] * curr_density[i])));
+          }
+          // use streaming writes to eliminate the memory reads from loading cache lines
+          copy_pixel_nontemporal(out + 4*(x+i), res);
+          length += 4*length_inc;
+        }
+      }
+      // handle the left-over pixels
+      for(int x = width & ~3; x < width; x++)
+      {
+        const float curr_density = _compute_density(density, length);
+        dt_aligned_pixel_t res;	// the compiler will optimize this into a register
+        for_each_channel(l, aligned(in : 16))
+        {
+          res[l] = MAX(zero[l], (in[4*x+l] * (color[l] + color1[l] * curr_density)));
+        }
+        // use streaming writes to eliminate the memory reads from loading cache lines
+        copy_pixel_nontemporal(out + 4*x, res);
         length += length_inc;
       }
     }
+    // ensure that the nontemporal writes have finished before continuing
+    dt_omploop_sfence();
   }
 
   if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK)
     dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
 }
 
-#if defined(__SSE__)
-void process_sse2(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const void *const ivoid,
-                  void *const ovoid, const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
-{
-  const dt_iop_graduatednd_data_t *const data = (const dt_iop_graduatednd_data_t *const)piece->data;
-  const int ch = piece->colors;
-
-  const int ix = (roi_in->x);
-  const int iy = (roi_in->y);
-  const float iw = piece->buf_in.width * roi_out->scale;
-  const float ih = piece->buf_in.height * roi_out->scale;
-  const float hw = iw / 2.0f;
-  const float hh = ih / 2.0f;
-  const float hw_inv = 1.0f / hw;
-  const float hh_inv = 1.0f / hh;
-  const float v = (-data->rotation / 180) * M_PI;
-  const float sinv = sinf(v);
-  const float cosv = cosf(v);
-  const float filter_radie = sqrtf((hh * hh) + (hw * hw)) / hh;
-  const float offset = data->offset / 100.0f * 2;
-
-  const float filter_hardness = 1.0 / filter_radie / (1.0 - (0.5 + (data->hardness / 100.0) * 0.9 / 2.0)) * 0.5;
-
-  const int width = roi_out->width;
-  const int height = roi_out->height;
-  if(data->density > 0)
-  {
-#ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    dt_omp_firstprivate(ch, cosv, data, filter_hardness, hh_inv, hw_inv, \
-                        ivoid, ix, iy, offset, ovoid, height, width, sinv)    \
-    schedule(static)
-#endif
-    for(int y = 0; y < height; y++)
-    {
-      const size_t k = (size_t)width * y * ch;
-      const float *const restrict in = (float *)ivoid + k;
-      float *const restrict out = (float *)ovoid + k;
-
-      float length = (sinv * (-1.0 + ix * hw_inv) - cosv * (-1.0 + (iy + y) * hh_inv) - 1.0 + offset)
-                     * filter_hardness;
-      const float length_inc = sinv * hw_inv * filter_hardness;
-
-      const __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
-      const __m128 c1 = _mm_set1_ps(1.0f) - c;
-
-      for(int x = 0; x < width; x++)
-      {
-        const __m128 density = _mm_set1_ps(compute_density(data->density, length));
-
-        /* max(0,in / (c + (1-c)*density)) */
-        _mm_stream_ps(out + ch*x, _mm_max_ps(_mm_set1_ps(0.0f), (_mm_load_ps(in + ch*x) / (c + (c1 * density)))));
-        length += length_inc;
-      }
-    }
-  }
-  else
-  {
-#ifdef _OPENMP
-#pragma omp parallel for default(none) \
-    dt_omp_firstprivate(ch, cosv, data, filter_hardness, hh_inv, hw_inv, \
-                        ivoid, ix, iy, offset, ovoid, height, width, sinv)    \
-    schedule(static)
-#endif
-    for(int y = 0; y < height; y++)
-    {
-      const size_t k = (size_t)width * y * ch;
-      const float *const restrict in = (float *)ivoid + k;
-      float *const restrict out = (float *)ovoid + k;
-
-      float length = (sinv * (-1.0f + ix * hw_inv) - cosv * (-1.0f + (iy + y) * hh_inv) - 1.0f + offset)
-                      * filter_hardness;
-      const float length_inc = sinv * hw_inv * filter_hardness;
-
-      const __m128 c = _mm_set_ps(0, data->color[2], data->color[1], data->color[0]);
-      const __m128 c1 = _mm_set1_ps(1.0f) - c;
-
-      for(int x = 0; x < width; x++)
-      {
-        const __m128 density = _mm_set1_ps(compute_density(-data->density, -length));
-
-        /* max(0,in * (c + (1-c)*density)) */
-        _mm_stream_ps(out + ch*x, _mm_max_ps(_mm_set1_ps(0.0f), (_mm_load_ps(in + ch*x) * (c + (c1 * density)))));
-        length += length_inc;
-      }
-    }
-  }
-  _mm_sfence();
-
-  if(piece->pipe->mask_display & DT_DEV_PIXELPIPE_DISPLAY_MASK) dt_iop_alpha_copy(ivoid, ovoid, roi_out->width, roi_out->height);
-}
-#endif
-
-
 #ifdef HAVE_OPENCL
-int process_cl(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, cl_mem dev_in, cl_mem dev_out,
-               const dt_iop_roi_t *const roi_in, const dt_iop_roi_t *const roi_out)
+int process_cl(struct dt_iop_module_t *self,
+               dt_dev_pixelpipe_iop_t *piece,
+               cl_mem dev_in, cl_mem dev_out,
+               const dt_iop_roi_t *const roi_in,
+               const dt_iop_roi_t *const roi_out)
 {
   dt_iop_graduatednd_data_t *data = (dt_iop_graduatednd_data_t *)piece->data;
   dt_iop_graduatednd_global_data_t *gd = (dt_iop_graduatednd_global_data_t *)self->global_data;
@@ -1031,16 +1048,18 @@ void gui_changed(dt_iop_module_t *self, GtkWidget *w, void *previous)
   dt_iop_graduatednd_gui_data_t *g = (dt_iop_graduatednd_gui_data_t *)self->gui_data;
   if(w == g->rotation)
   {
-    set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset);
+    _set_points_from_grad(self, &g->xa, &g->ya, &g->xb, &g->yb, p->rotation, p->offset);
   }
   else if(w == g->hue)
   {
-    update_saturation_slider_end_color(g->saturation, p->hue);
+    _update_saturation_slider_end_color(g->saturation, p->hue);
     gtk_widget_queue_draw(g->saturation);
   }
 }
 
-void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe,
+void commit_params(struct dt_iop_module_t *self,
+                   dt_iop_params_t *p1,
+                   dt_dev_pixelpipe_t *pipe,
                    dt_dev_pixelpipe_iop_t *piece)
 {
   dt_iop_graduatednd_params_t *p = (dt_iop_graduatednd_params_t *)p1;
@@ -1060,12 +1079,16 @@ void commit_params(struct dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pix
   for(int l = 0; l < 4; l++) d->color1[l] = 1.0 - d->color[l];
 }
 
-void init_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+void init_pipe(struct dt_iop_module_t *self,
+               dt_dev_pixelpipe_t *pipe,
+               dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = calloc(1, sizeof(dt_iop_graduatednd_data_t));
 }
 
-void cleanup_pipe(struct dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
+void cleanup_pipe(struct dt_iop_module_t *self,
+                  dt_dev_pixelpipe_t *pipe,
+                  dt_dev_pixelpipe_iop_t *piece)
 {
   free(piece->data);
   piece->data = NULL;
@@ -1079,7 +1102,7 @@ void gui_update(struct dt_iop_module_t *self)
   dt_iop_color_picker_reset(self, TRUE);
 
   g->define = 0;
-  update_saturation_slider_end_color(g->saturation, p->hue);
+  _update_saturation_slider_end_color(g->saturation, p->hue);
 }
 
 void gui_init(struct dt_iop_module_t *self)
