@@ -1,6 +1,6 @@
 /*
     This file is part of darktable,
-    Copyright (C) 2020-2022 darktable developers.
+    Copyright (C) 2020-2023 darktable developers.
 
     darktable is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with darktable.  If not, see <http://www.gnu.org/licenses/>.
 */
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -40,7 +41,7 @@ DT_MODULE_INTROSPECTION(1, dt_iop_sigmoid_params_t)
 typedef enum dt_iop_sigmoid_methods_type_t
 {
   DT_SIGMOID_METHOD_PER_CHANNEL = 0,     // $DESCRIPTION: "per channel"
-  DT_SIGMOID_METHOD_RGB_RATIO = 1,     // $DESCRIPTION: "rgb ratio"
+  DT_SIGMOID_METHOD_RGB_RATIO = 1,     // $DESCRIPTION: "RGB ratio"
 } dt_iop_sigmoid_methods_type_t;
 
 
@@ -75,6 +76,13 @@ typedef struct dt_iop_sigmoid_gui_data_t
 
 } dt_iop_sigmoid_gui_data_t;
 
+typedef struct dt_iop_sigmoid_global_data_t
+{
+  int kernel_sigmoid_loglogistic_per_channel;
+  int kernel_sigmoid_loglogistic_per_channel_interpolated;
+  int kernel_sigmoid_loglogistic_rgb_ratio;
+} dt_iop_sigmoid_global_data_t;
+
 
 const char *name()
 {
@@ -114,6 +122,28 @@ int default_colorspace(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_p
 
 void init_presets(dt_iop_module_so_t *self)
 {
+  // auto-applied scene-referred default
+  self->pref_based_presets = TRUE;
+
+  const char *workflow = dt_conf_get_string_const("plugins/darkroom/workflow");
+  const gboolean auto_apply_sigmoid = strcmp(workflow, "scene-referred (sigmoid)") == 0;
+
+  if(auto_apply_sigmoid)
+  {
+    dt_gui_presets_add_generic
+      (_("scene-referred default"), self->op, self->version(),
+       NULL, 0,
+       1, DEVELOP_BLEND_CS_RGB_SCENE);
+
+    dt_gui_presets_update_ldr(_("scene-referred default"), self->op,
+                              self->version(), FOR_RAW | FOR_MATRIX);
+
+    dt_gui_presets_update_autoapply(_("scene-referred default"),
+                                    self->op, self->version(), TRUE);
+  }
+
+  // others
+
   dt_iop_sigmoid_params_t p;
   p.display_white_target = 100.0f;
   p.display_black_target = 0.0152f;
@@ -132,7 +162,7 @@ void init_presets(dt_iop_module_so_t *self)
   p.middle_grey_contrast = 1.0f;
   p.contrast_skewness = 0.0f;
   p.color_processing = DT_SIGMOID_METHOD_RGB_RATIO;
-  dt_gui_presets_add_generic(_("reinhard"), self->op, self->version(), &p, sizeof(p), 1, DEVELOP_BLEND_CS_RGB_SCENE);
+  dt_gui_presets_add_generic(_("Reinhard"), self->op, self->version(), &p, sizeof(p), 1, DEVELOP_BLEND_CS_RGB_SCENE);
 }
 
 // Declared here as it is used in the commit params function
@@ -150,7 +180,7 @@ static inline float generalized_loglogistic_sigmoid(const float value, const flo
   const float paper_response = magnitude * powf(film_response / (paper_exp + film_response), paper_power);
 
   // Safety check for very large floats that cause numerical errors
-  return isnan(paper_response) ? magnitude : paper_response;
+  return dt_isnan(paper_response) ? magnitude : paper_response;
 }
 
 void commit_params(dt_iop_module_t *self, dt_iop_params_t *p1, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
@@ -517,6 +547,100 @@ void process(struct dt_iop_module_t *self, dt_dev_pixelpipe_iop_t *piece, const
   }
 }
 
+#ifdef HAVE_OPENCL
+int process_cl(struct dt_iop_module_t *self,
+               dt_dev_pixelpipe_iop_t *piece,
+               cl_mem dev_in,
+               cl_mem dev_out,
+               const dt_iop_roi_t *const roi_in,
+               const dt_iop_roi_t *const roi_out)
+{
+  const dt_iop_sigmoid_data_t *const d = (dt_iop_sigmoid_data_t *)piece->data;
+  dt_iop_sigmoid_global_data_t *const gd = (dt_iop_sigmoid_global_data_t *)self->global_data;
+
+  cl_int err = DT_OPENCL_DEFAULT_ERROR;
+  const int devid = piece->pipe->devid;
+  const int width = roi_in->width;
+  const int height = roi_in->height;
+
+  const float white_target = d->white_target;
+  const float paper_exp = d->paper_exposure;
+  const float film_fog = d->film_fog;
+  const float contrast_power = d->film_power;
+  const float skew_power = d->paper_power;
+
+  if (d->color_processing == DT_SIGMOID_METHOD_PER_CHANNEL)
+  {
+    const float hue_preservation = d->hue_preservation;
+
+    if (hue_preservation >= 0.001f)
+    {
+
+      err = dt_opencl_enqueue_kernel_2d_args(devid, gd->kernel_sigmoid_loglogistic_per_channel_interpolated,
+                                             width, height,
+                                             CLARG(dev_in), CLARG(dev_out), CLARG(width), CLARG(height),
+                                             CLARG(white_target), CLARG(paper_exp),
+                                             CLARG(film_fog), CLARG(contrast_power),
+                                             CLARG(skew_power), CLARG(hue_preservation));
+      if(err != CL_SUCCESS) goto error;
+      return TRUE;
+    }
+    else
+    {
+      err = dt_opencl_enqueue_kernel_2d_args(devid, gd->kernel_sigmoid_loglogistic_per_channel,
+                                             width, height,
+                                             CLARG(dev_in), CLARG(dev_out), CLARG(width), CLARG(height),
+                                             CLARG(white_target), CLARG(paper_exp),
+                                             CLARG(film_fog), CLARG(contrast_power),
+                                             CLARG(skew_power));
+      if(err != CL_SUCCESS) goto error;
+      return TRUE;
+
+    }
+  }
+  else
+  {
+    const float black_target = d->black_target;
+
+    err = dt_opencl_enqueue_kernel_2d_args(devid, gd->kernel_sigmoid_loglogistic_rgb_ratio,
+                                           width, height,
+                                           CLARG(dev_in), CLARG(dev_out), CLARG(width), CLARG(height),
+                                           CLARG(white_target), CLARG(black_target), CLARG(paper_exp),
+                                           CLARG(film_fog), CLARG(contrast_power),
+                                           CLARG(skew_power));
+    if(err != CL_SUCCESS) goto error;
+    return TRUE;
+  }
+
+  error:
+  dt_print(DT_DEBUG_OPENCL, "[opencl_sigmoid] couldn't enqueue kernel! %s\n", cl_errstr(err));
+  return FALSE;
+}
+#endif //HAVE_OPENCL
+
+void init_global(dt_iop_module_so_t *module)
+{
+  const int program = 36; // sigmoid.cl, from programs.conf
+  dt_iop_sigmoid_global_data_t *gd  =
+    (dt_iop_sigmoid_global_data_t *)malloc(sizeof(dt_iop_sigmoid_global_data_t));
+
+  module->data = gd;
+  gd->kernel_sigmoid_loglogistic_per_channel = dt_opencl_create_kernel(program, "sigmoid_loglogistic_per_channel");
+  gd->kernel_sigmoid_loglogistic_per_channel_interpolated =
+    dt_opencl_create_kernel(program, "sigmoid_loglogistic_per_channel_interpolated");
+  gd->kernel_sigmoid_loglogistic_rgb_ratio = dt_opencl_create_kernel(program, "sigmoid_loglogistic_rgb_ratio");
+}
+
+void cleanup_global(dt_iop_module_so_t *module)
+{
+  dt_iop_sigmoid_global_data_t *gd = (dt_iop_sigmoid_global_data_t *)module->data;
+  dt_opencl_free_kernel(gd->kernel_sigmoid_loglogistic_per_channel);
+  dt_opencl_free_kernel(gd->kernel_sigmoid_loglogistic_per_channel_interpolated);
+  dt_opencl_free_kernel(gd->kernel_sigmoid_loglogistic_rgb_ratio);
+  free(module->data);
+  module->data = NULL;
+}
+
 void init_pipe(dt_iop_module_t *self, dt_dev_pixelpipe_t *pipe, dt_dev_pixelpipe_iop_t *piece)
 {
   piece->data = calloc(1, sizeof(dt_iop_sigmoid_data_t));
@@ -576,7 +700,8 @@ void gui_init(dt_iop_module_t *self)
     (&g->cs,
      "plugins/darkroom/sigmoid/expand_values",
      _("display luminance"),
-     GTK_BOX(self->widget));
+     GTK_BOX(self->widget),
+     DT_ACTION(self));
   gtk_widget_set_tooltip_text(g->cs.expander,
                                 _("set display black/white targets"));
   GtkWidget *main_box = self->widget;
